@@ -18,13 +18,17 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
+type profile
+
 val f :
   ?standalone:bool ->
+  ?profile:profile ->
   ?toplevel:bool ->
   ?linkall:bool ->
   ?source_map:(string * Source_map.t) ->
-  Pretty_print.t -> Parse_bytecode.debug_loc -> Code.program -> unit
+  Pretty_print.t -> Parse_bytecode.Debug.data -> Code.program -> unit
 
 val from_string : string array -> string -> Pretty_print.t -> unit
 
-val set_profile : int -> unit
+val profiles : (int * profile) list
+val profile : int -> profile option
