@@ -18,4 +18,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-val f : Code.program -> int array -> Javascript.program
+val f :
+  Code.program -> ?toplevel:bool -> int array -> Parse_bytecode.Debug.data ->
+  Javascript.program
