@@ -19,7 +19,7 @@
  *)
 
 type t =
-    ACC0
+  | ACC0
   | ACC1
   | ACC2
   | ACC3
@@ -111,8 +111,6 @@ type t =
   | PUSHTRAP
   | POPTRAP
   | RAISE
-  | RERAISE
-  | RAISE_NOTRACE
   | CHECK_SIGNALS
   | C_CALL1
   | C_CALL2
@@ -165,6 +163,12 @@ type t =
   | GETPUBMET
   | GETDYNMET
   | STOP
+  | EVENT
+  | BREAK
+  | RERAISE
+  | RAISE_NOTRACE
+  | FIRST_UNIMPLEMENTED_OP
+
 
 type kind =
   | KNullary
@@ -176,13 +180,22 @@ type kind =
   | KSwitch
   | KClosurerec
   | KClosure
+  | KNullaryCall
+  | KUnaryCall
+  | KBinaryCall
   | KStop of int
+  | K_will_not_happen
 
 type desc = { code : t; kind : kind; name : string; opcode : int }
 
 let ops,ops_rev =
   let ops_rev = Hashtbl.create 17 in
-  let block1 =
+  let if_v4 =
+    match Util.Version.v with
+      `V3    -> (fun _ default -> default)
+    | `V4_02 -> (fun k _ -> k)
+  in
+  let instrs =
     [| ACC0, KNullary, "ACC0";
        ACC1, KNullary, "ACC1";
        ACC2, KNullary, "ACC2";
@@ -215,10 +228,10 @@ let ops,ops_rev =
        PUSHENVACC4, KNullary, "PUSHENVACC4";
        PUSHENVACC, KUnary, "PUSHENVACC";
        PUSH_RETADDR, KUnary, "PUSH_RETADDR";
-       APPLY, KUnary, "APPLY";
-       APPLY1, KNullary, "APPLY1";
-       APPLY2, KNullary, "APPLY2";
-       APPLY3, KNullary, "APPLY3";
+       APPLY, KUnaryCall, "APPLY";
+       APPLY1, KNullaryCall, "APPLY1";
+       APPLY2, KNullaryCall, "APPLY2";
+       APPLY3, KNullaryCall, "APPLY3";
        APPTERM, KStop 2, "APPTERM";
        APPTERM1, KStop 1, "APPTERM1";
        APPTERM2, KStop 1, "APPTERM2";
@@ -274,18 +287,14 @@ let ops,ops_rev =
        BOOLNOT, KNullary, "BOOLNOT";
        PUSHTRAP, KCond_jump, "PUSHTRAP";
        POPTRAP, KNullary, "POPTRAP";
-       RAISE, KStop 0, "RAISE" |]
-  and block2 =
-    [| RERAISE, KStop 0, "RERAISE";
-       RAISE_NOTRACE, KStop 0, "RAISE_NOTRACE" |]
-  and block3 =
-    [| CHECK_SIGNALS, KNullary, "CHECK_SIGNALS";
-       C_CALL1, KUnary, "C_CALL1";
-       C_CALL2, KUnary, "C_CALL2";
-       C_CALL3, KUnary, "C_CALL3";
-       C_CALL4, KUnary, "C_CALL4";
-       C_CALL5, KUnary, "C_CALL5";
-       C_CALLN, KBinary, "C_CALLN";
+       RAISE, KStop 0, "RAISE";
+       CHECK_SIGNALS, KNullary, "CHECK_SIGNALS";
+       C_CALL1, KUnaryCall, "C_CALL1";
+       C_CALL2, KUnaryCall, "C_CALL2";
+       C_CALL3, KUnaryCall, "C_CALL3";
+       C_CALL4, KUnaryCall, "C_CALL4";
+       C_CALL5, KUnaryCall, "C_CALL5";
+       C_CALLN, KBinaryCall, "C_CALLN";
        CONST0, KNullary, "CONST0";
        CONST1, KNullary, "CONST1";
        CONST2, KNullary, "CONST2";
@@ -330,19 +339,18 @@ let ops,ops_rev =
        BUGEINT, KCmp_jump, "BUGEINT";
        GETPUBMET, KBinary, "GETPUBMET";
        GETDYNMET, KNullary, "GETDYNMET";
-       STOP, KStop 0, "STOP"|]
-  in
-  let instrs =
-    match Util.Version.v with
-      `V3    -> [block1; block3]
-    | `V4_02 -> [block1; block2; block3]
-  in
+       STOP, KStop 0, "STOP";
+       EVENT, K_will_not_happen, "EVENT";
+       BREAK, K_will_not_happen, "BREAK";
+       RERAISE, if_v4 (KStop 0) K_will_not_happen, "RERAISE";
+       RAISE_NOTRACE, if_v4 (KStop 0) K_will_not_happen, "RAISE_NOTRACE";
+       FIRST_UNIMPLEMENTED_OP, K_will_not_happen, "FIRST_UNIMPLEMENTED_OP"|] in
   let ops =
     Array.mapi
       (fun i (c, k, n) ->
          Hashtbl.add ops_rev c i;
-         {code = c; kind = k; name = n; opcode = i})
-      (Array.concat instrs)
+         {code = c; kind = k; name = n; opcode = i}
+      ) instrs
   in
   ops,ops_rev
 
@@ -354,16 +362,19 @@ let int_to_buf buf i =
   Buffer.add_char buf (Char.chr ((i lsr 16) land 0xFF));
   Buffer.add_char buf (Char.chr ((i lsr 24) land 0xFF))
 
-let compile l =
-  let b = Buffer.create 50 in
+let compile b l =
   List.iter (fun i ->
       let i = match i with
         | `C i -> i
         | `I i -> to_int i in
-      int_to_buf b i) l;
+      int_to_buf b i) l
+
+let compile_to_string l =
+  let b = Buffer.create 50 in
+  compile b l;
   Buffer.contents b
 
-let get code i = Char.code code.[i]
+let get code i = Char.code (code.[i])
 
 let getu code pc =
   let i = pc * 4 in
@@ -391,7 +402,9 @@ exception Bad_instruction of int
 let get_instr code pc =
   let i = getu code pc in
   if i < 0 || i >= Array.length ops then raise (Bad_instruction i);
-  ops.(i)
+  let ins = ops.(i) in
+  if ins.kind = K_will_not_happen then raise (Bad_instruction i);
+  ins
 
 (****)
 
