@@ -19,7 +19,7 @@
  *)
 
 type t =
-    ACC0
+  | ACC0
   | ACC1
   | ACC2
   | ACC3
@@ -111,8 +111,6 @@ type t =
   | PUSHTRAP
   | POPTRAP
   | RAISE
-  | RERAISE
-  | RAISE_NOTRACE
   | CHECK_SIGNALS
   | C_CALL1
   | C_CALL2
@@ -165,6 +163,11 @@ type t =
   | GETPUBMET
   | GETDYNMET
   | STOP
+  | EVENT
+  | BREAK
+  | RERAISE
+  | RAISE_NOTRACE
+  | FIRST_UNIMPLEMENTED_OP
 
 type kind =
   | KNullary
@@ -176,7 +179,11 @@ type kind =
   | KSwitch
   | KClosurerec
   | KClosure
+  | KNullaryCall
+  | KUnaryCall
+  | KBinaryCall
   | KStop of int
+  | K_will_not_happen
 
 type desc = { code : t; kind : kind; name : string; opcode : int }
 
@@ -189,6 +196,6 @@ val getu : string -> int -> int
 val gets32 : string -> int -> int32
 val getu32 : string -> int -> int32
 
-val compile : [`C of int | `I of t] list -> string
+val compile_to_string : [`C of int | `I of t] list -> string
 
 val print_obj : Format.formatter -> Obj.t -> unit
