@@ -37,11 +37,10 @@ module Label = struct
   let of_string s = S s
 end
 
-type loc = Code.DebugAddr.dbg
-type node_pc =
-  | Loc of loc
+type location =
   | Pi of Parse_info.t
   | N
+  | U
 
 type identifier = string
 
@@ -85,7 +84,7 @@ and expression =
   | ECond of expression * expression * expression
   | EBin of binop * expression * expression
   | EUn of unop * expression
-  | ECall of expression * arguments
+  | ECall of expression * arguments * location
   | EAccess of expression * expression
   | EDot of expression * identifier
   | ENew of expression * arguments option
@@ -104,24 +103,25 @@ and expression =
 (* A.4 Statements *)
 
 and statement =
-    Block of block * node_pc
+    Block of block
   | Variable_statement of variable_declaration list
-  | Empty_statement of node_pc
-  | Expression_statement of expression * node_pc
-  | If_statement of expression * statement * statement option * node_pc
-  | Do_while_statement of statement * expression * node_pc
-  | While_statement of expression * statement * node_pc
-  | For_statement of  (expression option,variable_declaration list) either * expression option * expression option * statement * node_pc
-  | ForIn_statement of  (expression,variable_declaration) either * expression * statement * node_pc
-  | Continue_statement of Label.t option * node_pc
-  | Break_statement of Label.t option * node_pc
-  | Return_statement of expression option * node_pc
+  | Empty_statement
+  | Expression_statement of expression 
+  | If_statement of expression * (statement * location) * (statement * location) option 
+  | Do_while_statement of (statement * location) * expression 
+  | While_statement of expression * (statement * location)
+  | For_statement of  (expression option,variable_declaration list) either * expression option * expression option * (statement * location)
+  | ForIn_statement of  (expression,variable_declaration) either * expression * (statement * location)
+  | Continue_statement of Label.t option 
+  | Break_statement of Label.t option 
+  | Return_statement of expression option 
   (* | With_statement of expression * statement *)
-  | Labelled_statement of Label.t * statement * node_pc
-  | Switch_statement of expression * case_clause list * statement_list option * node_pc
-  | Throw_statement of expression * node_pc
-  | Try_statement of block * (ident * block) option * block option * node_pc
-  | Debugger_statement of node_pc
+  | Labelled_statement of Label.t * (statement * location)
+  | Switch_statement of
+      expression * case_clause list * statement_list option * case_clause list
+  | Throw_statement of expression 
+  | Try_statement of block * (ident * block) option * block option 
+  | Debugger_statement
 
 and ('left,'right) either =
   | Left of 'left
@@ -129,23 +129,23 @@ and ('left,'right) either =
 
 and block = statement_list
 
-and statement_list = statement list
+and statement_list = (statement * location) list
 
 and variable_declaration = ident * initialiser option
 
 and case_clause = expression * statement_list
 
-and initialiser = expression * node_pc
+and initialiser = expression * location
 
 (****)
 
 (* A.5 Functions and programs *)
 
 and function_declaration =
-  ident * formal_parameter_list * function_body * node_pc
+  ident * formal_parameter_list * function_body * location
 
 and function_expression =
-  ident option * formal_parameter_list * function_body * node_pc
+  ident option * formal_parameter_list * function_body * location
 
 and formal_parameter_list = ident list
 
@@ -153,7 +153,7 @@ and function_body = source_elements
 
 and program = source_elements
 
-and source_elements = source_element list
+and source_elements = (source_element * location) list
 
 and source_element =
     Statement of statement
