@@ -16,7 +16,18 @@
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *)
+*)
 
-val add_file : string -> unit
-val resolve_deps : ?linkall:bool -> bool -> Pretty_print.t -> string list -> string list
+val parse_file : string ->
+  ((Parse_info.t option * string * Primitive.kind * Primitive.kind_arg list option) option * (* provide *)
+   string list * (* require *)
+   ((int -> int -> bool) * string) list list * (* version constraint *)
+   Javascript.program) list
+
+val load_files : string list -> unit
+
+type state
+val init : unit -> state
+val resolve_deps : ?linkall:bool -> state -> Util.StringSet.t -> state * Util.StringSet.t
+val link : Javascript.program -> state -> Javascript.program
+val get_provided : unit -> Util.StringSet.t
