@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-let times = Util.debug "times"
+let times = Option.Debug.find "times"
 
 open Code
 
