@@ -53,12 +53,12 @@ type t =
     output : string -> int -> int -> unit }
 
 let spaces = String.make 80 ' '
-let output st s l =
+let output st (s : string) l =
   (try
-     let last = String.rindex_from s (l-1) '\n' in
+     let last = String.rindex_from s (l-1) '\n' + 1 in
      let line = ref 0 in
      for i = 0 to l-1 do
-       if s.[i] = '\n' then incr line;
+       if String.get s i = '\n' then incr line;
      done;
      st.line <- st.line + !line;
      st.col <- l - last
@@ -143,7 +143,7 @@ let rec push st e =
 
 (****)
 
-let string st s =
+let string st (s : string) =
   if st.compact then (
     let len = (String.length s) in
     if len <> 0
@@ -220,12 +220,24 @@ let newline st =
   st.indent <- 0; st.box_indent <- 0; st.prev_indents <- [];
   st.cur <- 0; st.l <- []; st.n <- 0; st.w <- 0
 
+(* hack on*)
+let output_substring = Pervasives.output
+(* for ocaml <  4.02, output_substring will be Pervasives.ouput (above)
+   for ocaml >= 4.02, output_substring will be taken from the locally
+                      open Pervasives module *)
+let output_substring =
+  let open Pervasives in
+  output_substring
+(* hack off*)
+
+
 let to_out_channel ch =
   { indent = 0; box_indent = 0; prev_indents = [];
     limit = 78; cur = 0; l = []; n = 0; w = 0;
     col = 0; line = 0; total = 0;
     compact = false; pending_space = None; last_char = None; needed_space = None;
-    output = fun s i l -> Pervasives.output ch s i l }
+    output = output_substring ch
+  }
 
 let to_buffer b =
   { indent = 0; box_indent = 0; prev_indents = [];
