@@ -20,11 +20,18 @@
 
 type t
 
+type pos = {
+  mutable p_line : int;
+  mutable p_col : int
+}
+
 val string : t -> string -> unit
 
 val genbreak : t -> string -> int -> unit
 val break : t -> unit
-val space : t -> unit
+val break1 : t -> unit
+val non_breaking_space : t -> unit
+val space : ?indent:int -> t -> unit
 
 val start_group : t -> int -> unit
 val end_group : t -> unit
@@ -33,5 +40,8 @@ val newline : t -> unit
 
 val to_out_channel : out_channel -> t
 val to_buffer : Buffer.t -> t
+val pos : t -> pos
+val total : t -> int
 
 val set_compact : t -> bool -> unit
+val set_needed_space_function : t -> (char -> char -> bool) -> unit
