@@ -54,8 +54,10 @@ val raise_ : exn -> unit
 
 val split_char : char -> string -> string list
 val split : string -> string -> string list
-val find : string -> string -> int
+val find : string -> Bytes.t -> int
 
+(* [normalize_argv argv] returns a new array of arguments where '-long-option' are replaced by '--long-option' *)
+val normalize_argv : ?warn:bool -> string array -> string array
 
 module Version : sig
   type t = int list
