@@ -46,6 +46,7 @@ class type cssStyleDeclaration = object
   method borderLeftColor : js_string t prop
   method borderLeftStyle : js_string t prop
   method borderLeftWidth : js_string t prop
+  method borderRadius : js_string t prop 
   method borderRight : js_string t prop
   method borderRightColor : js_string t prop
   method borderRightStyle : js_string t prop
@@ -71,6 +72,7 @@ class type cssStyleDeclaration = object
   method direction : js_string t prop
   method display : js_string t prop
   method emptyCells : js_string t prop
+  method fill : js_string t prop
   method font : js_string t prop
   method fontFamily : js_string t prop
   method fontSize : js_string t prop
@@ -110,10 +112,14 @@ class type cssStyleDeclaration = object
   method paddingTop : js_string t prop
   method pageBreakAfter : js_string t prop
   method pageBreakBefore : js_string t prop
+  method pointerEvents : js_string t prop
   method position : js_string t prop
   method right : js_string t prop
+  method stroke : js_string t prop
+  method strokeWidth : js_string t prop
   method tableLayout : js_string t prop
   method textAlign : js_string t prop
+  method textAnchor : js_string t prop
   method textDecoration : js_string t prop
   method textIndent : js_string t prop
   method textTransform : js_string t prop
@@ -307,6 +313,8 @@ and element = object
   method style : cssStyleDeclaration t prop
 
   method innerHTML : js_string t prop
+  method outerHTML : js_string t prop
+  method textContent : js_string t opt prop
 
   method clientLeft : int readonly_prop
   method clientTop : int readonly_prop
@@ -383,6 +391,7 @@ module Event = struct
   let focus = Dom.Event.make "focus"
   let blur = Dom.Event.make "blur"
   let load = Dom.Event.make "load"
+  let unload = Dom.Event.make "unload"
   let beforeunload = Dom.Event.make "beforeunload"
   let resize = Dom.Event.make "resize"
   let orientationchange = Dom.Event.make "orientationchange"
@@ -404,6 +413,8 @@ module Event = struct
   let cached = Dom.Event.make "cached"
   let obsolete = Dom.Event.make "obsolete"
 
+  let domContentLoaded = Dom.Event.make "DOMContentLoaded"
+
   let make = Dom.Event.make
 end
 
@@ -494,7 +505,7 @@ class type optionElement = object
   method defaultSelected : bool t prop
   method text : js_string t readonly_prop
   method index : int readonly_prop
-  method selected : bool prop
+  method selected : bool t prop
   method value : js_string t prop
 end
 
@@ -1008,6 +1019,53 @@ end
 external pixel_get : canvasPixelArray t -> int -> int = "caml_js_get"
 external pixel_set : canvasPixelArray t -> int -> int -> unit = "caml_js_set"
 
+class type range = object
+  method collapsed : bool t readonly_prop
+  method startOffset : int readonly_prop
+  method endOffset : int readonly_prop
+  method startContainer : Dom.node t readonly_prop
+  method endContainer : Dom.node t readonly_prop
+  method setStart : Dom.node t -> int -> unit meth
+  method setEnd : Dom.node t -> int -> unit meth
+  method setStartBefore : Dom.node t -> unit meth
+  method setEndBefore : Dom.node t -> unit meth
+  method setStartAfter : Dom.node t -> unit meth
+  method setEndAfter : Dom.node t -> unit meth
+  method selectNode : Dom.node t -> unit meth
+  method selectNodeContents : Dom.node t -> unit meth
+  method collapse : bool t -> unit meth
+  method cloneContents : Dom.documentFragment t meth
+  method extactContents : Dom.documentFragment t meth
+  method deleteContents : unit meth
+  method insertNode : Dom.node t -> unit meth
+  method surroundContents : Dom.node t -> unit meth
+  method cloneRange : range t meth
+  method toString : js_string t meth
+end
+
+(** Information on current selection *)
+class type selection = object
+  method anchorNode : Dom.node t readonly_prop
+  method anchorOffset : int readonly_prop
+  method focusNode : Dom.node t readonly_prop
+  method focusOffset : int readonly_prop
+  method isCollapsed : bool t readonly_prop
+  method rangeCount : int readonly_prop
+  method getRangeAt : int -> range t meth
+  method collapse : bool t -> unit meth
+  method extend : Dom.node t -> int -> unit meth
+  method modify : js_string t -> js_string t -> js_string t -> unit meth
+  method collapseToStart : unit meth
+  method collapseToEnd : unit meth
+  method selectAllChildren : Dom.node t -> unit meth
+  method addRange : range t -> unit meth
+  method removeRange : range t -> unit meth
+  method removeAllRanges : unit meth
+  method deleteFromDocument : unit meth
+  method containsNode : Dom.node t -> bool t -> bool t meth
+  method toString : js_string t meth
+end
+
 class type document = object
   inherit [element] Dom.document
   inherit nodeSelector
@@ -1031,6 +1089,8 @@ class type document = object
   method close : unit meth
   method write : js_string t -> unit meth
   method execCommand : js_string t -> bool t -> js_string t opt -> unit meth
+  method createRange : range t meth
+  method readyState : js_string t readonly_prop
 
   inherit eventTarget
 end
@@ -1066,9 +1126,6 @@ end
 class type undoManager = object
 end
 
-class type selection = object
-end
-
 class type navigator = object
   method appCodeName : js_string t readonly_prop
   method appName : js_string t readonly_prop
@@ -1152,6 +1209,7 @@ class type window = object
   method outerHeight : int optdef readonly_prop
 
   method onload : (window t, event t) event_listener prop
+  method onunload : (window t, event t) event_listener prop
   method onbeforeunload : (window t, event t) event_listener prop
   method onblur : (window t, event t) event_listener prop
   method onfocus : (window t, event t) event_listener prop
@@ -1167,6 +1225,10 @@ end
 let window : window t = Js.Unsafe.global (* The toplevel object *)
 
 let document = window##document
+let getElementById id =
+  Js.Opt.case (document##getElementById (Js.string id))
+    (fun () -> raise Not_found)
+    (fun pnode -> pnode)
 
 (****)
 
