@@ -45,6 +45,7 @@ class type cssStyleDeclaration = object
   method borderLeftColor : js_string t prop
   method borderLeftStyle : js_string t prop
   method borderLeftWidth : js_string t prop
+  method borderRadius : js_string t prop 
   method borderRight : js_string t prop
   method borderRightColor : js_string t prop
   method borderRightStyle : js_string t prop
@@ -70,6 +71,7 @@ class type cssStyleDeclaration = object
   method direction : js_string t prop
   method display : js_string t prop
   method emptyCells : js_string t prop
+  method fill : js_string t prop
   method font : js_string t prop
   method fontFamily : js_string t prop
   method fontSize : js_string t prop
@@ -109,10 +111,14 @@ class type cssStyleDeclaration = object
   method paddingTop : js_string t prop
   method pageBreakAfter : js_string t prop
   method pageBreakBefore : js_string t prop
+  method pointerEvents : js_string t prop (* SVG-only on many browsers *)
   method position : js_string t prop
   method right : js_string t prop
+  method stroke : js_string t prop
+  method strokeWidth : js_string t prop
   method tableLayout : js_string t prop
   method textAlign : js_string t prop
+  method textAnchor : js_string t prop
   method textDecoration : js_string t prop
   method textIndent : js_string t prop
   method textTransform : js_string t prop
@@ -321,6 +327,8 @@ and element = object
   method style : cssStyleDeclaration t prop
 
   method innerHTML : js_string t prop
+  method outerHTML : js_string t prop
+  method textContent : js_string t opt prop
 
   method clientLeft : int readonly_prop
   method clientTop : int readonly_prop
@@ -441,7 +449,7 @@ class type optionElement = object
   method defaultSelected : bool t prop
   method text : js_string t readonly_prop
   method index : int readonly_prop
-  method selected : bool prop
+  method selected : bool t prop
   method value : js_string t prop
 end
 
@@ -959,6 +967,54 @@ end
 external pixel_get : canvasPixelArray t -> int -> int = "caml_js_get"
 external pixel_set : canvasPixelArray t -> int -> int -> unit = "caml_js_set"
 
+(** Object representing a range **)
+class type range = object
+  method collapsed : bool t readonly_prop
+  method startOffset : int readonly_prop
+  method endOffset : int readonly_prop
+  method startContainer : Dom.node t readonly_prop
+  method endContainer : Dom.node t readonly_prop
+  method setStart : Dom.node t -> int -> unit meth
+  method setEnd : Dom.node t -> int -> unit meth
+  method setStartBefore : Dom.node t -> unit meth
+  method setEndBefore : Dom.node t -> unit meth
+  method setStartAfter : Dom.node t -> unit meth
+  method setEndAfter : Dom.node t -> unit meth
+  method selectNode : Dom.node t -> unit meth
+  method selectNodeContents : Dom.node t -> unit meth
+  method collapse : bool t -> unit meth
+  method cloneContents : Dom.documentFragment t meth
+  method extactContents : Dom.documentFragment t meth
+  method deleteContents : unit meth
+  method insertNode : Dom.node t -> unit meth
+  method surroundContents : Dom.node t -> unit meth
+  method cloneRange : range t meth
+  method toString : js_string t meth
+end
+
+(** Information on current selection *)
+class type selection = object
+  method anchorNode : Dom.node t readonly_prop
+  method anchorOffset : int readonly_prop
+  method focusNode : Dom.node t readonly_prop
+  method focusOffset : int readonly_prop
+  method isCollapsed : bool t readonly_prop
+  method rangeCount : int readonly_prop
+  method getRangeAt : int -> range t meth
+  method collapse : bool t -> unit meth
+  method extend : Dom.node t -> int -> unit meth
+  method modify : js_string t -> js_string t -> js_string t -> unit meth
+  method collapseToStart : unit meth
+  method collapseToEnd : unit meth
+  method selectAllChildren : Dom.node t -> unit meth
+  method addRange : range t -> unit meth
+  method removeRange : range t -> unit meth
+  method removeAllRanges : unit meth
+  method deleteFromDocument : unit meth
+  method containsNode : Dom.node t -> bool t -> bool t meth
+  method toString : js_string t meth
+end
+
 (** {2 Document objects} *)
 
 class type document = object
@@ -984,12 +1040,18 @@ class type document = object
   method close : unit meth
   method write : js_string t -> unit meth
   method execCommand : js_string t -> bool t -> js_string t opt -> unit meth
+  method createRange : range t meth
+  method readyState : js_string t readonly_prop
 
   inherit eventTarget
 end
 
 val document : document t
-  (** The current document *)
+(** The current document *)
+
+val getElementById : string -> element Js.t
+(** [getElementById id] returns the element with the id [id] in the
+    current document. It raises [Not_found] if there are no such element *)
 
 (** {2 Window objects} *)
 
@@ -1025,11 +1087,6 @@ class type undoManager = object
 (*...*)
 end
 
-(** Information on current selection *)
-class type selection = object
-(*...*)
-end
-
 (** Navigator information *)
 class type navigator = object
   method appCodeName : js_string t readonly_prop
@@ -1117,6 +1174,7 @@ class type window = object
   method outerHeight : int optdef readonly_prop
 
   method onload : (window t, event t) event_listener prop
+  method onunload : (window t, event t) event_listener prop
   method onbeforeunload : (window t, event t) event_listener prop
   method onblur : (window t, event t) event_listener prop
   method onfocus : (window t, event t) event_listener prop
@@ -1229,6 +1287,7 @@ module Event : sig
   val focus : event t typ
   val blur : event t typ
   val load : event t typ
+  val unload : event t typ
   val beforeunload : event t typ
   val resize : event t typ
   val orientationchange : event t typ
@@ -1250,6 +1309,8 @@ module Event : sig
   val cached : event t typ
   val obsolete : event t typ
 
+  val domContentLoaded : event t typ
+
   val make : string -> 'a typ
 end
 
