@@ -22,7 +22,6 @@ type +'a t
 
 module Unsafe = struct
   external variable : string -> 'a = "caml_js_var"
-  external constant : string -> 'a = "caml_js_const"
 
   type any
   external inject : 'a -> any = "%identity"
@@ -44,7 +43,9 @@ module Unsafe = struct
 
   external eval_string : string -> 'a = "caml_js_eval_string"
 
-  external expr : string -> 'a = "caml_js_expr"
+  external js_expr : string -> 'a = "caml_js_expr"
+
+  external pure_js_expr : string -> 'a = "caml_pure_js_expr"
 
   let global = variable "joo_global_object"
 end
@@ -56,10 +57,10 @@ type 'a optdef = 'a
 
 external debugger : unit -> unit = "debugger"
 
-let null : 'a opt = Unsafe.constant "null"
+let null : 'a opt = Unsafe.pure_js_expr "null"
 external some : 'a -> 'a opt = "%identity"
 
-let undefined : 'a optdef = Unsafe.constant "undefined"
+let undefined : 'a optdef = Unsafe.pure_js_expr "undefined"
 external def : 'a -> 'a optdef = "%identity"
 
 module type OPT = sig
@@ -130,8 +131,8 @@ external wrap_meth_callback : ('a -> 'b -> 'c) -> ('a, 'b -> 'c) meth_callback =
 
 (****)
 
-let _true = Unsafe.constant "true"
-let _false = Unsafe.constant "false"
+let _true = Unsafe.pure_js_expr "true"
+let _false = Unsafe.pure_js_expr "false"
 
 type match_result_handle
 type string_array
@@ -218,6 +219,17 @@ class type ['a] js_array = object
   method unshift_2 : 'a -> 'a -> int meth
   method unshift_3 : 'a -> 'a -> 'a -> int meth
   method unshift_4 : 'a -> 'a -> 'a -> 'a -> int meth
+
+  method some : ('a -> int -> 'a js_array t -> bool t ) -> bool t meth
+  method every : ('a -> int -> 'a js_array t -> bool t ) -> bool t meth
+  method forEach : ('a -> int -> 'a js_array t -> unit ) -> unit meth
+  method map : ('a -> int -> 'a js_array t -> 'b) -> 'b js_array t meth
+  method filter : ('a -> int -> 'a js_array t -> bool t) -> 'a js_array t meth
+  method reduce_init : ('b -> 'a -> int -> 'a js_array t -> 'b) -> 'b -> 'b meth
+  method reduce : ('a -> 'a -> int -> 'a js_array t -> 'a) -> 'a meth
+  method reduceRight_init : ('b -> 'a -> int -> 'a js_array t -> 'b) -> 'b -> 'b meth
+  method reduceRight : ('a -> 'a -> int -> 'a js_array t -> 'a) -> 'a meth
+
   method length : int prop
 end
 
@@ -345,6 +357,12 @@ exception Error of error t
 let error_constr = Unsafe.global##_Error
 let _ = Callback.register_exception "jsError" (Error (Unsafe.obj [||]))
 
+class type json = object
+  method parse : js_string t -> 'a meth
+  method stringify: 'a -> js_string t meth
+end
+
+let _JSON : json t = Unsafe.global##_JSON
 
 
 let decodeURI (s : js_string t) : js_string t =
@@ -366,7 +384,7 @@ external string : string -> js_string t = "caml_js_from_string"
 external to_string : js_string t -> string = "caml_js_to_string"
 external array : 'a array -> 'a js_array t = "caml_js_from_array"
 external to_array : 'a js_array t -> 'a array = "caml_js_to_array"
-external bytestring : string -> js_string t = "caml_js_from_byte_string"
+external bytestring : string -> js_string t = "caml_bytes_of_string"
 external to_bytestring : js_string t -> string = "caml_js_to_byte_string"
 
 external typeof : < .. > t -> js_string t = "caml_js_typeof"
@@ -396,6 +414,8 @@ let _ =
        if instanceof e array_constructor then None
        else Some (to_string ((Obj.magic e)##toString())))
 
+let string_of_error e = to_string (e##toString())
+
 (****)
 
 (* DEPRECATED *)
