@@ -165,6 +165,7 @@ type string_array
       (This type is used to resolved the mutual dependency between
        string and array type definitions.) *)
 
+
 (** Specification of Javascript string objects. *)
 class type js_string = object
   method toString : js_string t meth
@@ -235,7 +236,9 @@ val regExp_copy : (regExp t -> regExp t) constr
       [jsnew regExp (r)] builds a copy of regular expression [r]. *)
 
 (** Specification of Javascript regular arrays. *)
+(** use [Js.array_get] and [Js.array_set] to access and set array elements. *)
 class type ['a] js_array = object
+
   method toString : js_string t meth
   method toLocaleString : js_string t meth
   method concat : 'a js_array t -> 'a js_array t meth
@@ -260,6 +263,17 @@ class type ['a] js_array = object
   method unshift_2 : 'a -> 'a -> int meth
   method unshift_3 : 'a -> 'a -> 'a -> int meth
   method unshift_4 : 'a -> 'a -> 'a -> 'a -> int meth
+
+  method some : ('a -> int -> 'a js_array t -> bool t ) -> bool t meth
+  method every : ('a -> int -> 'a js_array t -> bool t ) -> bool t meth
+  method forEach : ('a -> int -> 'a js_array t -> unit ) -> unit meth
+  method map : ('a -> int -> 'a js_array t -> 'b) -> 'b js_array t meth
+  method filter : ('a -> int -> 'a js_array t -> bool t) -> 'a js_array t meth
+  method reduce_init : ('b -> 'a -> int -> 'a js_array t -> 'b) -> 'b -> 'b meth
+  method reduce : ('a -> 'a -> int -> 'a js_array t -> 'a) -> 'a meth
+  method reduceRight_init : ('b -> 'a -> int -> 'a js_array t -> 'b) -> 'b -> 'b meth
+  method reduceRight : ('a -> 'a -> int -> 'a js_array t -> 'a) -> 'a meth
+
   method length : int prop
 end
 
@@ -427,12 +441,24 @@ val error_constr : (js_string t -> error t) constr
       [jsnew error_constr (msg)]
       returns an [Error] object with the message [msg]. *)
 
+val string_of_error : error t -> string
+
 exception Error of error t
   (** The [Error] exception wrap javascript exceptions when catched by ocaml code.
       In case the javascript exception is not an instance of javascript [Error],
       it will be serialized and wrapped into a [Failure] exception.
   *)
 
+
+(** Specification of Javascript JSON object. *)
+class type json = object
+  method parse : js_string t -> 'a meth
+  method stringify: 'a -> js_string t meth
+end
+
+val _JSON : json t
+(** JSON object *)
+
 (** {2 Standard Javascript functions} *)
 
 val decodeURI : js_string t -> js_string t
@@ -479,7 +505,7 @@ external array : 'a array -> 'a js_array t = "caml_js_from_array"
   (** Conversion of arrays from OCaml to Javascript. *)
 external to_array : 'a js_array t -> 'a array = "caml_js_to_array"
   (** Conversion of arrays from Javascript to OCaml. *)
-external bytestring : string -> js_string t = "caml_js_from_byte_string"
+external bytestring : string -> js_string t = "caml_bytes_of_string"
   (** Conversion of strings of bytes from OCaml to Javascript.
       (Each byte will be converted in an UTF-16 code point.) *)
 external to_bytestring : js_string t -> string = "caml_js_to_byte_string"
@@ -570,11 +596,14 @@ module Unsafe : sig
   external eval_string : string -> 'a = "caml_js_eval_string"
     (** Evaluate Javascript code *)
 
-  external expr : string -> 'a = "caml_js_expr"
-    (** [expr e] will parse the JavaScript expression [e] if [e]
+  external js_expr : string -> 'a = "caml_js_expr"
+    (** [js_expr e] will parse the JavaScript expression [e] if [e]
         is available at compile time or will failback to a
         runtime evaluation. See [eval_string] *)
 
+  external pure_js_expr : string -> 'a = "caml_pure_js_expr"
+    (** [pure_js_expr str] behaves like [pure_expr (fun () -> js_expr str)]. *)
+
   val global : < .. > t
     (** Javascript global object *)
 
