@@ -414,6 +414,8 @@ val request_animation_frame : unit -> unit Lwt.t
 (** Returns when the page is loaded *)
 val onload : unit -> Dom_html.event Js.t Lwt.t
 
+val domContentLoaded : unit -> unit Lwt.t
+val onunload : unit -> Dom_html.event Js.t Lwt.t
 val onbeforeunload : unit -> Dom_html.event Js.t Lwt.t
 val onresize : unit -> Dom_html.event Js.t Lwt.t
 val onorientationchange : unit -> Dom_html.event Js.t Lwt.t
