@@ -20,7 +20,9 @@
 
 let rnd = Random.State.make [|0x313511d4|]
 let random_var () =
-  Format.sprintf "a%08Lx" (Random.State.int64 rnd 0x100000000L)
+  Format.sprintf "x%08Lx" (Random.State.int64 rnd 0x100000000L)
+let random_tvar () =
+  Format.sprintf "A%08Lx" (Random.State.int64 rnd 0x100000000L)
 
 open Camlp4
 
@@ -90,40 +92,61 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
     with Not_found ->
       lab
 
-  let fresh_type _loc = <:ctyp< '$random_var ()$ >>
+  let fresh_type _loc = <:ctyp< '$random_tvar ()$ >>
 
-  let constrain_types _loc (e:string) (v:string) v_typ m m_typ =
+  let constrain_types
+          _loc e_loc (e:string) v_loc (v:string) v_typ m_loc m m_typ args =
+    let cstr =
+      let _loc = e_loc in <:expr<($lid:e$ : $js_t_id _loc "t"$ 'B)>> in
+    let x = let _loc = e_loc in <:expr<x>> in
+    let body =
+      let _loc = Syntax.Loc.merge e_loc m_loc in
+      <:expr<($x$#$lid:m$ : $m_typ$)>> in
+    let y = let _loc = v_loc in <:expr<($lid:v$ : $v_typ$)>> in
+    let res =
+      List.fold_right
+        (fun (e, x, t) e' ->
+           let _loc = Ast.loc_of_expr e in
+           <:expr< let _ = ($lid:x$ : $t$) in $e'$ >>)
+        args
+        <:expr<
+          $y$
+        >>
+    in
     <:expr<
       let module M = struct
-        value _ =
-          let _ign1 = ($lid:e$ : $js_t_id _loc "t"$ 'b) in
-          let _ign2 = ($lid:v$ : $v_typ$) in
-          (fun (x : 'b) -> (x#$lid:m$ : $m_typ$ ));
-        end
-      in ()
-    >>
+        value res =
+          let _ = $cstr$ in
+          let _ = fun (x : 'B) -> $body$ in
+          $res$;
+      end in M.res >>
 
   let method_call _loc obj lab lab_loc args =
-    let args = List.map (fun e -> (e, fresh_type _loc)) args in
+    let args = List.map (fun e -> (e, random_var (), fresh_type _loc)) args in
     let ret_type = fresh_type _loc in
     let method_type =
       List.fold_right
-        (fun (_, arg_ty) rem_ty -> <:ctyp< $arg_ty$ -> $rem_ty$ >>)
+        (fun (_, _, arg_ty) rem_ty -> <:ctyp< $arg_ty$ -> $rem_ty$ >>)
         args <:ctyp< $js_t_id _loc "meth"$ $ret_type$ >>
     in
     let o = random_var () in
-    let fakev = random_var () in
-    let typ = fresh_type _loc in
-    let args =
-      List.map (fun (e, t) -> <:expr< $js_u_id _loc "inject"$ $with_type e t$ >>) args
+    let res = random_var () in
+    let meth_args =
+      List.map (fun (_, x, _) -> <:expr< $js_u_id _loc "inject"$ $lid:x$ >>)
+        args
     in
-    let args = make_array _loc args in
-    <:expr<
-      let $lid:o$ = $obj$ in
-      let $lid:fakev$ : $typ$ = Obj.magic () in
-      let () = $constrain_types _loc o fakev ret_type lab method_type$ in
-      (($js_u_id _loc "meth_call"$ $lid:o$ $str:unescape lab$ $args$): $typ$)
-    >>
+    let meth_args = make_array _loc meth_args in
+    let o_loc = Ast.loc_of_expr obj in
+    List.fold_left
+      (fun e' (e, x, _) -> <:expr< let $lid:x$ = $e$ in $e'$>>)
+      <:expr<
+        let $lid:o$ = $obj$ in
+        let $lid:res$ =
+          $js_u_id _loc "meth_call"$ $lid:o$ $str:unescape lab$ $meth_args$ in
+        $constrain_types _loc o_loc o _loc res ret_type
+           lab_loc lab method_type args$
+      >>
+      args
 
   let new_object _loc constructor args =
     let args = List.map (fun e -> (e, fresh_type _loc)) args in
@@ -134,12 +157,14 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
         args obj_type
     in
     let args =
-      List.map (fun (e, t) -> <:expr< $js_u_id _loc "inject"$ $with_type e t$ >>) args
+      List.map
+        (fun (e, t) -> <:expr< $js_u_id _loc "inject"$ $with_type e t$ >>) args
     in
     let args = make_array _loc args in
     let x = random_var () in
     let constr =
-      with_type constructor <:ctyp< $js_t_id _loc "constr"$ $constr_fun_type$ >> in
+      with_type constructor
+        <:ctyp< $js_t_id _loc "constr"$ $constr_fun_type$ >> in
     with_type
       <:expr< let $lid:x$ = $constr$ in
               $js_u_id _loc "new_obj"$ $lid:x$ $args$ >>
@@ -153,21 +178,25 @@ module Make (Syntax : Sig.Camlp4Syntax) = struct
     ["##" RIGHTA
      [ e = SELF; (lab_loc, lab) = jsmeth ->
        let o = random_var () in
-       let fakev = random_var () in
-       let typ = fresh_type _loc in
+       let o_loc = Ast.loc_of_expr e in
+       let res = random_var () in
        <:expr<
          let $lid:o$ = $e$ in
-         let $lid:fakev$ : $typ$ = Obj.magic () in
-         let () = $constrain_types _loc o fakev <:ctyp< 'a >> lab <:ctyp< $js_t_id _loc "gen_prop"$ <get : 'a; ..> >>$ in
-         ($js_u_id _loc "get"$ $lid:o$ $str:unescape lab$ : $typ$)
-         >>
+         let $lid:res$ = $js_u_id _loc "get"$ $lid:o$ $str:unescape lab$ in
+         $constrain_types _loc o_loc o _loc res <:ctyp< 'A >>
+            lab_loc lab <:ctyp< $js_t_id _loc "gen_prop"$ <get : 'A; ..> >> []$
+       >>
      | e1 = SELF; (lab_loc, lab) = jsmeth; "<-"; e2 = expr LEVEL "top" ->
        let o = random_var () in
+       let o_loc = Ast.loc_of_expr e1 in
        let v = random_var () in
        <:expr<
-         let $lid:o$ = $e1$ in
          let $lid:v$ = $e2$ in
-         let () = $constrain_types _loc o v <:ctyp< 'a >> lab <:ctyp< $js_t_id _loc "gen_prop"$ <set : 'a -> unit; ..> >>$ in
+         let $lid:o$ = $e1$ in
+         let _ = $constrain_types _loc o_loc o (Ast.loc_of_expr e2) v
+                    <:ctyp< 'A >> lab_loc lab
+                    <:ctyp< $js_t_id _loc "gen_prop"$ <set : 'A -> unit; ..> >>
+                    []$ in
          $js_u_id _loc "set"$ $lid:o$ $str:unescape lab$ ($lid:v$)
          >>
      | e = SELF; (lab_loc, lab) = jsmeth; "("; ")" ->
