@@ -18,6 +18,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
+(** WebGL binding *)
+
 open Js
 
 (** 5.1 Types *)
@@ -26,6 +28,7 @@ type sizeiptr = int
 type intptr = int
 type uint = int
 type clampf = float
+type void
 
 type clearBufferMask = int
 type beginMode
@@ -48,12 +51,12 @@ type attachmentPoint
 type rbTarget
 type texTarget
 type 'a parameter
-type bufferParameter
+type 'a bufferParameter
 type 'a vertexAttribParam
 type vertexAttribPointerParam
 type 'a attachParam
 type framebufferStatus
-type renderbufferParam
+type 'a renderbufferParam
 type format
 type pixelFormat
 type pixelType
@@ -67,6 +70,8 @@ type wrapMode
 type texFilter
 type uniformType
 type colorspaceConversion
+type shaderPrecisionType
+type objectType
 
 (** 5.2 WebGLContextAttributes *)
 class type contextAttributes = object
@@ -76,8 +81,12 @@ class type contextAttributes = object
   method antialias : bool t prop
   method premultipliedAlpha : bool t prop
   method preserveDrawingBuffer : bool t prop
+  method preferLowPowerToHighPerformance : bool t prop
+  method failIfMajorPerformanceCaveat : bool t prop
 end
 
+val defaultContextAttributes : contextAttributes t
+
 type buffer
 type framebuffer
 type program
@@ -92,6 +101,12 @@ class type activeInfo = object
   method name : js_string t readonly_prop
 end
 
+class type shaderPrecisionFormat = object
+  method rangeMin : int readonly_prop
+  method rangeMax : int readonly_prop
+  method precision : int readonly_prop
+end
+
 class type renderingContext = object
   (** 5.13.1 Attributes *)
   method canvas : Dom_html.canvasElement t readonly_prop
@@ -144,7 +159,7 @@ class type renderingContext = object
 
   (** 5.13.5 Buffer objects *)
   method bindBuffer : bufferTarget -> buffer t -> unit meth
-  method bindBuffer_ : bufferTarget opt -> buffer t -> unit meth
+  method bindBuffer_ : bufferTarget -> buffer t opt -> unit meth
   method bufferData_create :
     bufferTarget -> sizeiptr -> bufferUsage -> unit meth
   method bufferData :
@@ -157,12 +172,12 @@ class type renderingContext = object
     bufferTarget -> intptr -> Typed_array.arrayBuffer t -> unit meth
   method createBuffer : buffer t meth
   method deleteBuffer : buffer t -> unit meth
-  method getBufferParameter : bufferTarget -> bufferParameter -> int meth
+  method getBufferParameter : 'a. bufferTarget -> 'a bufferParameter -> 'a meth
   method isBuffer : buffer t -> bool t meth
 
   (** 5.13.6 Framebuffer objects *)
   method bindFramebuffer : fbTarget -> framebuffer t -> unit meth
-  method bindFramebuffer_ : fbTarget opt -> framebuffer t -> unit meth
+  method bindFramebuffer_ : fbTarget -> framebuffer t opt -> unit meth
   method checkFramebufferStatus : fbTarget -> framebufferStatus meth
   method createFramebuffer : framebuffer t meth
   method deleteFramebuffer : framebuffer t -> unit meth
@@ -176,17 +191,24 @@ class type renderingContext = object
 
   (** 5.13.7 Renderbuffer objects *)
   method bindRenderbuffer : rbTarget -> renderbuffer t -> unit meth
-  method bindRenderbuffer_ : rbTarget opt -> renderbuffer t -> unit meth
+  method bindRenderbuffer_ : rbTarget -> renderbuffer t opt -> unit meth
   method createRenderbuffer : renderbuffer t meth
   method deleteRenderbuffer : renderbuffer t -> unit meth
-  method getRenderbufferParameter : rbTarget -> renderbufferParam -> int meth
+  method getRenderbufferParameter :
+    'a. rbTarget -> 'a renderbufferParam -> 'a meth
   method isRenderbuffer : renderbuffer t -> bool t meth
   method renderbufferStorage :
     rbTarget -> format -> sizei -> sizei -> unit meth
 
   (** 5.13.8 Texture objects *)
   method bindTexture : texTarget -> texture t -> unit meth
-  method bindTexture_ : texTarget opt -> texture t -> unit meth
+  method bindTexture_ : texTarget -> texture t opt -> unit meth
+  method compressedTexImage2D :
+    texTarget -> int -> pixelFormat -> sizei -> sizei -> int ->
+    #Typed_array.arrayBufferView t -> unit meth
+  method compressedTexSubImage2D :
+    texTarget -> int -> int -> int -> sizei -> sizei -> pixelFormat ->
+    #Typed_array.arrayBufferView t -> unit meth
   method copyTexImage2D :
     texTarget -> int -> pixelFormat -> int -> int -> sizei -> sizei ->
     int -> unit meth
@@ -197,9 +219,12 @@ class type renderingContext = object
   method generateMipmap : texTarget -> unit meth
   method getTexParameter : texTarget -> 'a texParam -> 'a meth
   method isTexture : texture t -> bool t meth
+  method texImage2D_new :
+    texTarget -> int -> pixelFormat -> sizei -> sizei -> int ->
+    pixelFormat -> pixelType -> void opt -> unit meth
   method texImage2D_fromView :
     texTarget -> int -> pixelFormat -> sizei -> sizei -> int ->
-    pixelFormat -> pixelType -> #Typed_array.arrayBufferView t -> unit
+    pixelFormat -> pixelType -> #Typed_array.arrayBufferView t -> unit meth
   method texImage2D_fromImageData :
     texTarget -> int -> pixelFormat -> pixelFormat -> pixelType ->
     Dom_html.imageData t -> unit meth
@@ -245,6 +270,8 @@ class type renderingContext = object
   method getProgramParameter : 'a . program t -> 'a programParam -> 'a meth
   method getProgramInfoLog : program t -> js_string t meth
   method getShaderParameter : 'a . shader t -> 'a shaderParam -> 'a meth
+  method getShaderPrecisionFormat :
+    shaderType -> shaderPrecisionType -> shaderPrecisionFormat t meth
   method getShaderInfoLog : shader t -> js_string t meth
   method getShaderSource : shader t -> js_string t meth
   method isProgram : program t -> bool t meth
@@ -252,7 +279,6 @@ class type renderingContext = object
   method linkProgram : program t -> unit meth
   method shaderSource : shader t -> js_string t -> unit meth
   method useProgram : program t -> unit meth
-  method useProgram_ : program t opt -> unit meth
   method validateProgram : program t -> unit meth
 
   (** 5.13.10 Uniforms and attributes *)
@@ -355,7 +381,7 @@ class type renderingContext = object
 
   (** 5.13.14 Detecting and enabling extensions *)
   method getSupportedExtensions : js_string t js_array t meth
-  method getExtension : 'a. js_string t -> 'a t meth (* Untyped! *)
+  method getExtension : 'a. js_string t -> 'a t opt meth (* Untyped! *)
 
   (** Constants *)
 
@@ -409,21 +435,21 @@ class type renderingContext = object
   method _DEPTH_TEST_ : enableCap readonly_prop
   method _SCISSOR_TEST_ : enableCap readonly_prop
   method _POLYGON_OFFSET_FILL_ : enableCap readonly_prop
-  method _SAMPLE_ALPHA_TO_COVERAGE : enableCap readonly_prop
-  method _SAMPLE_COVERAGE : enableCap readonly_prop
+  method _SAMPLE_ALPHA_TO_COVERAGE_ : enableCap readonly_prop
+  method _SAMPLE_COVERAGE_ : enableCap readonly_prop
 
-  method _NO_ERROR : errorCode readonly_prop
-  method _INVALID_ENUM : errorCode readonly_prop
-  method _INVALID_VALUE : errorCode readonly_prop
-  method _INVALID_OPERATION : errorCode readonly_prop
-  method _OUT_OF_MEMORY : errorCode readonly_prop
-  method _CONTEXT_LOST_WEBGL : errorCode readonly_prop
+  method _NO_ERROR_ : errorCode readonly_prop
+  method _INVALID_ENUM_ : errorCode readonly_prop
+  method _INVALID_VALUE_ : errorCode readonly_prop
+  method _INVALID_OPERATION_ : errorCode readonly_prop
+  method _OUT_OF_MEMORY_ : errorCode readonly_prop
+  method _CONTEXT_LOST_WEBGL_ : errorCode readonly_prop
   method _INVALID_FRAMEBUFFER_OPERATION_ : errorCode readonly_prop
 
   method _CW : frontFaceDir readonly_prop
   method _CCW : frontFaceDir readonly_prop
 
-  method _DONT_CARE : hintMode readonly_prop
+  method _DONT_CARE_ : hintMode readonly_prop
   method _FASTEST : hintMode readonly_prop
   method _NICEST : hintMode readonly_prop
   method _GENERATE_MIPMAP_HINT_ : hintTarget readonly_prop
@@ -496,11 +522,12 @@ class type renderingContext = object
   method _SAMPLE_COVERAGE_VALUE_ : float parameter readonly_prop
   method _SAMPLE_COVERAGE_INVERT_ : bool t parameter readonly_prop
   method _NUM_COMPRESSED_TEXTURE_FORMATS_ : int parameter readonly_prop
-  method _COMPRESSED_TEXTURE_FORMATS_ : unit opt parameter readonly_prop
+  method _COMPRESSED_TEXTURE_FORMATS_ :
+    Typed_array.uint32Array t parameter readonly_prop
   method _GENERATE_MIPMAP_HINT_PARAM_ : hintMode parameter readonly_prop
 
-  method _BUFFER_SIZE_ : bufferParameter readonly_prop
-  method _BUFFER_USAGE_ : bufferParameter readonly_prop
+  method _BUFFER_SIZE_ : int bufferParameter readonly_prop
+  method _BUFFER_USAGE_ : bufferUsage bufferParameter readonly_prop
 
   method _BYTE : dataType readonly_prop
   method _UNSIGNED_BYTE_DT : dataType readonly_prop
@@ -534,7 +561,7 @@ class type renderingContext = object
   method _MAX_TEXTURE_IMAGE_UNITS_ : int parameter readonly_prop
   method _MAX_FRAGMENT_UNIFORM_VECTORS_ : int parameter readonly_prop
 
-  method _SHADER_TYPE_ : int shaderParam readonly_prop
+  method _SHADER_TYPE_ : shaderType shaderParam readonly_prop
   method _DELETE_STATUS_ : bool t shaderParam readonly_prop
   method _COMPILE_STATUS_ : bool t shaderParam readonly_prop
 
@@ -551,7 +578,7 @@ class type renderingContext = object
   method _RENDERER : js_string t parameter readonly_prop
   method _VERSION : js_string t parameter readonly_prop
   method _MAX_CUBE_MAP_TEXTURE_SIZE_ : int parameter readonly_prop
-  method _ACTIVE_TEXTURE_ : int parameter readonly_prop
+  method _ACTIVE_TEXTURE_ : textureUnit parameter readonly_prop
   method _FRAMEBUFFER_BINDING_ : framebuffer t opt parameter readonly_prop
   method _RENDERBUFFER_BINDING_ : renderbuffer t opt parameter readonly_prop
   method _MAX_RENDERBUFFER_SIZE : int parameter readonly_prop
@@ -586,9 +613,9 @@ class type renderingContext = object
   method _TEXTURE_WRAP_S_ : wrapMode texParam readonly_prop
   method _TEXTURE_WRAP_T_ : wrapMode texParam readonly_prop
 
-(*???
-  method _TEXTURE : enum readonly_prop
-*)
+  method _NONE_OT : objectType readonly_prop
+  method _TEXTURE_OT : objectType readonly_prop
+  method _RENDERBUFFER_OT : objectType readonly_prop
 
   method _TEXTURE_2D_ : texTarget readonly_prop
   method _TEXTURE_CUBE_MAP_ : texTarget readonly_prop
@@ -640,10 +667,11 @@ class type renderingContext = object
   method _FLOAT_VEC2_ : uniformType readonly_prop
   method _FLOAT_VEC3_ : uniformType readonly_prop
   method _FLOAT_VEC4_ : uniformType readonly_prop
+  method _INT_ : uniformType readonly_prop
   method _INT_VEC2_ : uniformType readonly_prop
   method _INT_VEC3_ : uniformType readonly_prop
   method _INT_VEC4_ : uniformType readonly_prop
-  method _BOOL : uniformType readonly_prop
+  method _BOOL_ : uniformType readonly_prop
   method _BOOL_VEC2_ : uniformType readonly_prop
   method _BOOL_VEC3_ : uniformType readonly_prop
   method _BOOL_VEC4_ : uniformType readonly_prop
@@ -665,14 +693,12 @@ class type renderingContext = object
   method _CURRENT_VERTEX_ATTRIB_ :
     Typed_array.float32Array t vertexAttribParam readonly_prop
 
-  (*??? Shader Precision-Specified Types
-  method _LOW_FLOAT_ : enum readonly_prop
-  method _MEDIUM_FLOAT_ : enum readonly_prop
-  method _HIGH_FLOAT_ : enum readonly_prop
-  method _LOW_INT_ : enum readonly_prop
-  method _MEDIUM_INT_ : enum readonly_prop
-  method _HIGH_INT_ : enum readonly_prop
-  *)
+  method _LOW_FLOAT_ : shaderPrecisionType readonly_prop
+  method _MEDIUM_FLOAT_ : shaderPrecisionType readonly_prop
+  method _HIGH_FLOAT_ : shaderPrecisionType readonly_prop
+  method _LOW_INT_ : shaderPrecisionType readonly_prop
+  method _MEDIUM_INT_ : shaderPrecisionType readonly_prop
+  method _HIGH_INT_ : shaderPrecisionType readonly_prop
 
   method _FRAMEBUFFER : fbTarget readonly_prop
   method _RENDERBUFFER : rbTarget readonly_prop
@@ -683,17 +709,19 @@ class type renderingContext = object
   method _DEPTH_COMPONENT16_ : format readonly_prop
   method _STENCIL_INDEX8_ : format readonly_prop
 
-  method _RENDERBUFFER_WIDTH_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_HEIGHT_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_INTERNAL_FORMAT_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_RED_SIZE_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_GREEN_SIZE_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_BLUE_SIZE_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_ALPHA_SIZE_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_DEPTH_SIZE_ : renderbufferParam readonly_prop
-  method _RENDERBUFFER_STENCIL_SIZE_ : renderbufferParam readonly_prop
+  method _RENDERBUFFER_WIDTH_ : int renderbufferParam readonly_prop
+  method _RENDERBUFFER_HEIGHT_ : int renderbufferParam readonly_prop
+  method _RENDERBUFFER_INTERNAL_FORMAT_ :
+    format renderbufferParam readonly_prop
+  method _RENDERBUFFER_RED_SIZE_ : int renderbufferParam readonly_prop
+  method _RENDERBUFFER_GREEN_SIZE_ : int renderbufferParam readonly_prop
+  method _RENDERBUFFER_BLUE_SIZE_ : int renderbufferParam readonly_prop
+  method _RENDERBUFFER_ALPHA_SIZE_ : int renderbufferParam readonly_prop
+  method _RENDERBUFFER_DEPTH_SIZE_ : int renderbufferParam readonly_prop
+  method _RENDERBUFFER_STENCIL_SIZE_ : int renderbufferParam readonly_prop
 
-  method _FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_ : int attachParam readonly_prop
+  method _FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE_ :
+    objectType attachParam readonly_prop
   method _FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_RENDERBUFFER :
     renderbuffer t attachParam readonly_prop
   method _FRAMEBUFFER_ATTACHMENT_OBJECT_NAME_TEXTURE :
@@ -709,7 +737,8 @@ class type renderingContext = object
 
   method _FRAMEBUFFER_COMPLETE_ : framebufferStatus readonly_prop
   method _FRAMEBUFFER_INCOMPLETE_ATTACHMENT_ : framebufferStatus readonly_prop
-  method _FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_ : framebufferStatus readonly_prop
+  method _FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT_ :
+    framebufferStatus readonly_prop
   method _FRAMEBUFFER_INCOMPLETE_DIMENSIONS_ : framebufferStatus readonly_prop
   method _FRAMEBUFFER_UNSUPPORTED_ : framebufferStatus readonly_prop
 
@@ -734,8 +763,6 @@ end
 class type contextEvent = object
   inherit Dom_html.event
   method statusMessage : js_string t readonly_prop
-  method initWebGLContextEvent :
-    js_string t -> bool t -> bool t -> js_string t -> unit meth
 end
 
 module Event : sig
@@ -747,3 +774,5 @@ end
 (** Get a context *)
 
 val getContext : Dom_html.canvasElement t -> renderingContext t opt
+val getContextWithAttributes :
+  Dom_html.canvasElement t -> contextAttributes t -> renderingContext t opt
