@@ -19,16 +19,16 @@
 
 
 //Provides: caml_md5_chan
-//Requires: caml_md5_string, MlStringFromArray
+//Requires: caml_md5_string, caml_string_of_array,caml_ml_string_length
 function caml_md5_chan(chan,len){
   if(len<0){
-    len=chan.data.array.length - chan.data.offset;
+    len=caml_ml_string_length(chan.file.data) - chan.offset;
   }
-  return caml_md5_string(new MlStringFromArray(chan.data.array),chan.data.offset,len);
+  return caml_md5_string(chan.file.data,chan.offset,len);
 }
 
 //Provides: caml_md5_string
-//Requires: MlString, MlStringFromArray
+//Requires: caml_string_of_array, caml_convert_string_to_bytes
 var caml_md5_string =
 function () {
   function add (x, y) { return (x + y) | 0; }
@@ -132,7 +132,7 @@ function () {
       w[3] = add(d, w[3]);
     }
 
-    var t = [];
+    var t = new Array(16);
     for (var i = 0; i < 4; i++)
       for (var j = 0; j < 4; j++)
         t[i * 4 + j] = (w[i] >> (8 * j)) & 0xFF;
@@ -143,15 +143,11 @@ function () {
     // FIX: maybe we should perform the computation by chunk of 64 bytes
     // as in http://www.myersdaily.org/joseph/javascript/md5.js
     var buf = [];
-    if (s.array) {
-      var a = s.array;
-      for (var i = 0; i < len; i+=4) {
-        var j = i + ofs;
-        buf[i>>2] = a[j] | (a[j+1] << 8) | (a[j+2] << 16) | (a[j+3] << 24);
-      }
-      for (; i < len; i++) buf[i>>2] |= a[i + ofs] << (8 * (i & 3));
-    } else {
-      var b = s.getFullBytes();
+    switch (s.t & 6) {
+    default:
+      caml_convert_string_to_bytes(s);
+    case 0: /* BYTES */
+      var b = s.c;
       for (var i = 0; i < len; i+=4) {
         var j = i + ofs;
         buf[i>>2] =
@@ -159,7 +155,15 @@ function () {
           (b.charCodeAt(j+2) << 16) | (b.charCodeAt(j+3) << 24);
       }
       for (; i < len; i++) buf[i>>2] |= b.charCodeAt(i + ofs) << (8 * (i & 3));
+      break;
+    case 4: /* ARRAY */
+      var a = s.c;
+      for (var i = 0; i < len; i+=4) {
+        var j = i + ofs;
+        buf[i>>2] = a[j] | (a[j+1] << 8) | (a[j+2] << 16) | (a[j+3] << 24);
+      }
+      for (; i < len; i++) buf[i>>2] |= a[i + ofs] << (8 * (i & 3));
     }
-    return new MlStringFromArray(md5(buf, len));
+    return caml_string_of_array(md5(buf, len));
   }
 } ();
