@@ -18,27 +18,68 @@
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
 ///////////// Core
+
+//Provides: raw_array_sub
+function raw_array_sub (a,i,l) {
+  var b = new Array(l);
+  for(var j = 0; j < l; j++) b[j] = a[i+j];
+  return b
+}
+
+//Provides: raw_array_copy
+function raw_array_copy (a) {
+  var l = a.length;
+  var b = new Array(l);
+  for(var i = 0; i < l; i++ ) b[i] = a[i];
+  return b
+}
+
+//Provides: raw_array_cons
+function raw_array_cons (a,x) {
+  var l = a.length;
+  var b = new Array(l+1);
+  b[0]=x;
+  for(var i = 1; i <= l; i++ ) b[i] = a[i-1];
+  return b
+}
+
+//Provides: raw_array_append_one
+function raw_array_append_one(a,x) {
+  var l = a.length;
+  var b = new Array(l+1);
+  var i = 0;
+  for(; i < l; i++ ) b[i] = a[i];
+  b[i]=x;
+  return b
+}
+
 //Provides: caml_call_gen
+//Requires: raw_array_sub
+//Requires: raw_array_append_one
 function caml_call_gen(f, args) {
   if(f.fun)
     return caml_call_gen(f.fun, args);
   var n = f.length;
-  var d = n - args.length;
+  var argsLen = args.length;
+  var d = n - argsLen;
   if (d == 0)
     return f.apply(null, args);
   else if (d < 0)
-    return caml_call_gen(f.apply(null, args.slice(0,n)), args.slice(n));
+    return caml_call_gen(f.apply(null,
+                                 raw_array_sub(args,0,n)),
+                         raw_array_sub(args,n,argsLen - n));
   else
-    return function (x){ return caml_call_gen(f, args.concat([x])); };
+    return function (x){ return caml_call_gen(f, raw_array_append_one(args,x)); };
 }
 
 //Provides: caml_named_values
 var caml_named_values = {};
 
 //Provides: caml_register_named_value
-//Requires: caml_named_values
+//Requires: caml_named_values, caml_bytes_of_string
 function caml_register_named_value(nm,v) {
-  caml_named_values[nm.toString()] = v; return 0;
+  caml_named_values[caml_bytes_of_string(nm)] = v;
+  return 0;
 }
 
 //Provides: caml_named_value
@@ -54,7 +95,7 @@ var caml_global_data = [0];
 //Requires: caml_global_data
 function caml_register_global (n, v) { caml_global_data[n + 1] = v; }
 
-//Provides: caml_get_global_data
+//Provides: caml_get_global_data mutable
 //Requires: caml_global_data
 function caml_get_global_data () { return caml_global_data; }
 
@@ -73,9 +114,9 @@ function caml_raise_constant (tag) { throw tag; }
 function caml_raise_with_arg (tag, arg) { throw [0, tag, arg]; }
 
 //Provides: caml_raise_with_string
-//Requires: caml_raise_with_arg,MlWrappedString
+//Requires: caml_raise_with_arg,caml_new_string
 function caml_raise_with_string (tag, msg) {
-  caml_raise_with_arg (tag, new MlWrappedString (msg));
+  caml_raise_with_arg (tag, caml_new_string (msg));
 }
 
 //Provides: caml_raise_sys_error
@@ -90,7 +131,7 @@ function caml_failwith (msg) {
   caml_raise_with_string(caml_global_data[3], msg);
 }
 //Provides: caml_wrap_exception
-//Requires: caml_global_data,MlWrappedString,caml_named_value
+//Requires: caml_global_data,caml_js_to_string,caml_named_value
 function caml_wrap_exception(e) {
   if(e instanceof Array) return e;
   //Stack_overflow: chrome, safari
@@ -109,7 +150,7 @@ function caml_wrap_exception(e) {
   if(e instanceof joo_global_object.Error)
     return [0,caml_named_value("jsError"),e];
   //fallback: wrapped in Failure
-  return [0,caml_global_data[3],new MlWrappedString (String(e))];
+  return [0,caml_global_data[3],caml_js_to_string (String(e))];
 }
 
 //Provides: caml_invalid_argument
@@ -152,17 +193,24 @@ function caml_update_dummy (x, y) {
 //Provides: caml_obj_is_block const
 function caml_obj_is_block (x) { return +(x instanceof Array); }
 //Provides: caml_obj_tag const
-function caml_obj_tag (x) { return (x instanceof Array)?x[0]:1000; }
+//Requires: MlString
+function caml_obj_tag (x) { return (x instanceof Array)?x[0]:(x instanceof MlString)?252:1000; }
 //Provides: caml_obj_set_tag
 function caml_obj_set_tag (x, tag) { x[0] = tag; return 0; }
 //Provides: caml_obj_block const
 function caml_obj_block (tag, size) {
-  var o = [tag];
+  var o = new Array(size+1);
+  o[0]=tag;
   for (var i = 1; i <= size; i++) o[i] = 0;
   return o;
 }
 //Provides: caml_obj_dup mutable
-function caml_obj_dup (x) { return x.slice(); }
+function caml_obj_dup (x) {
+  var l = x.length;
+  var a = new Array(l);
+  for(var i = 0; i < l; i++ ) a[i] = x[i];
+  return a;
+}
 //Provides: caml_obj_truncate
 function caml_obj_truncate (x, s) { x.length = s + 1; return 0; }
 
@@ -211,13 +259,24 @@ function caml_array_get (array, index) {
   return array[index+1];
 }
 
+//Provides: caml_check_bound
+//Requires: caml_array_bound_error
+function caml_check_bound (array, index) {
+  if (index >>> 0 >= array.length - 1) caml_array_bound_error();
+  return array;
+}
+
 //Provides: caml_make_vect const
 function caml_make_vect (len, init) {
-  var b = [0]; for (var i = 1; i <= len; i++) b[i] = init; return b;
+  var len = len + 1 | 0;
+  var b = new Array(len);
+  b[0]=0;
+  for (var i = 1; i < len; i++) b[i] = init;
+  return b;
 }
 
 //Provides: caml_compare_val
-//Requires: MlString, caml_int64_compare, caml_int_compare
+//Requires: MlString, caml_int64_compare, caml_int_compare, caml_string_compare
 //Requires: caml_invalid_argument
 function caml_compare_val (a, b, total) {
   var stack = [];
@@ -226,7 +285,7 @@ function caml_compare_val (a, b, total) {
       if (a instanceof MlString) {
         if (b instanceof MlString) {
             if (a !== b) {
-		var x = a.compare(b);
+		var x = caml_string_compare(a, b);
 		if (x != 0) return x;
 	    }
         } else
@@ -325,11 +384,12 @@ function caml_lessequal (x, y) { return +(caml_compare_val(x,y,false) <= 0); }
 function caml_lessthan (x, y) { return +(caml_compare_val(x,y,false) < 0); }
 
 //Provides: caml_parse_sign_and_base
-//Requires: MlString
+//Requires: caml_string_unsafe_get, caml_ml_string_length
 function caml_parse_sign_and_base (s) {
-  var i = 0, base = 10, sign = s.get(0) == 45?(i++,-1):1;
-  if (s.get(i) == 48)
-    switch (s.get(i + 1)) {
+  var i = 0, len = caml_ml_string_length(s), base = 10,
+     sign = (len > 0 && caml_string_unsafe_get(s,0) == 45)?(i++,-1):1;
+  if (i + 1 < len && caml_string_unsafe_get(s, i) == 48)
+    switch (caml_string_unsafe_get(s, i + 1)) {
     case 120: case 88: base = 16; i += 2; break;
     case 111: case 79: base =  8; i += 2; break;
     case  98: case 66: base =  2; i += 2; break;
@@ -346,25 +406,26 @@ function caml_parse_digit(c) {
 }
 
 //Provides: caml_int_of_string mutable
-//Requires: caml_parse_sign_and_base, caml_parse_digit, MlString, caml_failwith
+//Requires: caml_ml_string_length, caml_string_unsafe_get
+//Requires: caml_parse_sign_and_base, caml_parse_digit, caml_failwith
 function caml_int_of_string (s) {
   var r = caml_parse_sign_and_base (s);
   var i = r[0], sign = r[1], base = r[2];
+  var len = caml_ml_string_length(s);
   var threshold = -1 >>> 0;
-  var c = s.get(i);
+  var c = (i < len)?caml_string_unsafe_get(s, i):0;
   var d = caml_parse_digit(c);
   if (d < 0 || d >= base) caml_failwith("int_of_string");
   var res = d;
-  for (;;) {
-    i++;
-    c = s.get(i);
+  for (i++;i<len;i++) {
+    c = caml_string_unsafe_get(s, i);
     if (c == 95) continue;
     d = caml_parse_digit(c);
     if (d < 0 || d >= base) break;
     res = base * res + d;
     if (res > threshold) caml_failwith("int_of_string");
   }
-  if (i != s.getLen()) caml_failwith("int_of_string");
+  if (i != len) caml_failwith("int_of_string");
   // For base different from 10, we expect an unsigned representation,
   // hence any value of 'res' (less than 'threshold') is acceptable.
   // But we have to convert the result back to a signed integer.
@@ -376,10 +437,10 @@ function caml_int_of_string (s) {
 }
 
 //Provides: caml_float_of_string mutable
-//Requires: caml_failwith
+//Requires: caml_failwith, caml_bytes_of_string
 function caml_float_of_string(s) {
   var res;
-  s = s.getFullBytes();
+  s = caml_bytes_of_string (s);
   res = +s;
   if ((s.length > 0) && (res === res)) return res;
   s = s.replace(/_/g,"");
@@ -400,9 +461,9 @@ function caml_is_printable(c) { return +(c > 31 && c < 127); }
 
 ///////////// Format
 //Provides: caml_parse_format
-//Requires: caml_invalid_argument
+//Requires: caml_bytes_of_string, caml_invalid_argument
 function caml_parse_format (fmt) {
-  fmt = fmt.toString ();
+  fmt = caml_bytes_of_string(fmt);
   var len = fmt.length;
   if (len > 31) caml_invalid_argument("format_int: format too long");
   var f =
@@ -456,7 +517,7 @@ function caml_parse_format (fmt) {
 }
 
 //Provides: caml_finish_formatting
-//Requires: MlWrappedString
+//Requires: caml_new_string
 function caml_finish_formatting(f, rawbuffer) {
   if (f.uppercase) rawbuffer = rawbuffer.toUpperCase();
   var len = rawbuffer.length;
@@ -481,13 +542,14 @@ function caml_finish_formatting(f, rawbuffer) {
   buffer += rawbuffer;
   if (f.justify == '-')
     for (var i = len; i < f.width; i++) buffer += ' ';
-  return new MlWrappedString (buffer);
+  return caml_new_string (buffer);
 }
 
 //Provides: caml_format_int const
-//Requires: caml_parse_format, caml_finish_formatting, MlWrappedString,caml_str_repeat
+//Requires: caml_parse_format, caml_finish_formatting, caml_str_repeat
+//Requires: caml_new_string, caml_bytes_of_string
 function caml_format_int(fmt, i) {
-  if (fmt.toString() == "%d") return new MlWrappedString(""+i);
+  if (caml_bytes_of_string(fmt) == "%d") return caml_new_string(""+i);
   var f = caml_parse_format(fmt);
   if (i < 0) { if (f.signedconv) { f.sign = -1; i = -i; } else i >>>= 0; }
   var s = i.toString(f.base);
@@ -550,7 +612,8 @@ function caml_format_float (fmt, x) {
 
 ///////////// Hashtbl
 //Provides: caml_hash_univ_param mutable
-//Requires: MlString, caml_int64_to_bytes, caml_int64_bits_of_float
+//Requires: MlString, caml_convert_string_to_bytes
+//Requires: caml_int64_to_bytes, caml_int64_bits_of_float
 function caml_hash_univ_param (count, limit, obj) {
   var hash_accu = 0;
   function hash_aux (obj) {
@@ -578,13 +641,16 @@ function caml_hash_univ_param (count, limit, obj) {
       }
     } else if (obj instanceof MlString) {
       count --;
-      var a = obj.array, l = obj.getLen ();
-      if (a) {
-        for (var i = 0; i < l; i++) hash_accu = (hash_accu * 19 + a[i]) | 0;
-      } else {
-        var b = obj.getFullBytes ();
-        for (var i = 0; i < l; i++)
+      switch (obj.t & 6) {
+      default: /* PARTIAL */
+        caml_convert_string_to_bytes(obj);
+      case 0: /* BYTES */
+        for (var b = obj.c, l = obj.l, i = 0; i < l; i++)
           hash_accu = (hash_accu * 19 + b.charCodeAt(i)) | 0;
+        break;
+      case 2: /* ARRAY */
+        for (var a = obj.c, l = obj.l, i = 0; i < l; i++)
+          hash_accu = (hash_accu * 19 + a[i]) | 0;
       }
     } else if (obj === (obj|0)) {
       // Integer
@@ -602,24 +668,25 @@ function caml_hash_univ_param (count, limit, obj) {
 }
 
 //Provides: caml_hash mutable
-//Requires: MlString, caml_int64_bits_of_float, caml_mul
+//Requires: MlString, caml_convert_string_to_bytes
+//Requires: caml_int64_bits_of_float, caml_mul
 var caml_hash =
 function () {
   var HASH_QUEUE_SIZE = 256;
   function ROTL32(x,n) { return ((x << n) | (x >>> (32-n))); }
   function MIX(h,d) {
-    d = caml_mul(d, 0xcc9e2d51);
+    d = caml_mul(d, 0xcc9e2d51|0);
     d = ROTL32(d, 15);
     d = caml_mul(d, 0x1b873593);
     h ^= d;
     h = ROTL32(h, 13);
-    return ((((h * 5)|0) + 0xe6546b64)|0);
+    return (((h + (h << 2))|0) + (0xe6546b64|0))|0;
   }
   function FINAL_MIX(h) {
     h ^= h >>> 16;
-    h = caml_mul (h, 0x85ebca6b);
+    h = caml_mul (h, 0x85ebca6b|0);
     h ^= h >>> 13;
-    h = caml_mul (h, 0xc2b2ae35);
+    h = caml_mul (h, 0xc2b2ae35|0);
     h ^= h >>> 16;
     return h;
   }
@@ -711,12 +778,14 @@ function () {
           break;
         }
       } else if (v instanceof MlString) {
-        var a = v.array;
-        if (a) {
-          h = caml_hash_mix_string_arr(h, a);
-        } else {
-          var b = v.getFullBytes ();
-          h = caml_hash_mix_string_str(h, b);
+        switch (v.t & 6) {
+        default:
+          caml_convert_string_to_bytes (v);
+        case 0: /* BYTES */
+          h = caml_hash_mix_string_str(h, v.c);
+          break;
+        case 2: /* ARRAY */
+          h = caml_hash_mix_string_arr(h, v.c);
         }
         num--;
         break;
@@ -741,9 +810,9 @@ function () {
 var caml_initial_time = new Date() * 0.001;
 function caml_sys_time () { return new Date() * 0.001 - caml_initial_time; }
 //Provides: caml_sys_get_config const
-//Requires: MlWrappedString
+//Requires: caml_new_string
 function caml_sys_get_config () {
-  return [0, new MlWrappedString("Unix"), 32, 0];
+  return [0, caml_new_string("Unix"), 32, 0];
 }
 //Provides: caml_sys_random_seed mutable
 //The function needs to return an array since OCaml 4.0...
@@ -768,15 +837,32 @@ function caml_sys_const_ostype_unix () { return 1; }
 //Provides: caml_sys_const_ostype_win32 const
 function caml_sys_const_ostype_win32 () { return 0; }
 
+//Provides: caml_sys_system_command
+function caml_sys_system_command(_cmd){
+  return 127;
+}
+
 ///////////// Array
 //Provides: caml_array_sub mutable
 function caml_array_sub (a, i, len) {
-  return [0].concat(a.slice(i+1, i+1+len));
+  var a2 = new Array(len+1);
+  a2[0]=0;
+  for(var i2 = 1, i1= i+1; i2 <= len; i2++,i1++ ){
+    a2[i2]=a[i1];
+  }
+  return a2;
 }
 
 //Provides: caml_array_append mutable
 function caml_array_append(a1, a2) {
-  return a1.concat(a2.slice(1));
+  var l1 = a1.length, l2 = a2.length;
+  var l = l1+l2-1
+  var a = new Array(l);
+  a[0] = 0;
+  var i = 1,j = 1;
+  for(;i<l1;i++) a[i]=a1[i];
+  for(;i<l;i++,j++) a[i]=a2[j];
+  return a;
 }
 
 //Provides: caml_array_concat mutable
@@ -796,7 +882,8 @@ function caml_array_blit(a1, i1, a2, i2, len) {
     for (var j = 1; j <= len; j++) a2[i2 + j] = a1[i1 + j];
   } else {
     for (var j = len; j >= 1; j--) a2[i2 + j] = a1[i1 + j];
-  }
+  };
+  return 0;
 }
 
 ///////////// CamlinternalOO
@@ -843,7 +930,7 @@ function caml_convert_raw_backtrace () { return 0; }
 function caml_get_current_callstack () { return 0; }
 //Provides: caml_sys_getenv
 //Requires: caml_raise_not_found
-//Requires: MlWrappedString
+//Requires: caml_js_to_string
 function caml_sys_getenv (name) {
   var g = joo_global_object;
   var n = name.toString();
@@ -851,7 +938,7 @@ function caml_sys_getenv (name) {
   if(g.process
      && g.process.env
      && g.process.env[n] != undefined)
-    return new MlWrappedString(g.process.env[n]);
+    return caml_js_to_string(g.process.env[n]);
   caml_raise_not_found ();
 }
 //Provides: caml_sys_exit
@@ -865,7 +952,8 @@ function caml_sys_exit (code) {
 }
 
 //Provides: caml_sys_get_argv const
-//Requires: MlWrappedString
+//Requires: caml_js_to_string
+//Requires: raw_array_sub
 function caml_sys_get_argv () {
   var g = joo_global_object;
   var main = "a.out";
@@ -874,15 +962,16 @@ function caml_sys_get_argv () {
   if(g.process
      && g.process.argv
      && g.process.argv.length > 0) {
+    var argv = g.process.argv
     //nodejs
-    main = g.process.argv[0];
-    args = g.process.argv.slice(1);
+    main = argv[1];
+    args = raw_array_sub(argv,2,argv.length - 2);
   }
 
-  var p = new MlWrappedString(main);
+  var p = caml_js_to_string(main);
   var args2 = [0, p];
   for(var i = 0; i < args.length; i++)
-    args2.push(new MlWrappedString(args[i]));
+    args2.push(caml_js_to_string(args[i]));
   return [0, p, args2];
 }
 
@@ -899,3 +988,10 @@ function caml_set_oo_id (b) {
 
 //Provides: caml_install_signal_handler const
 function caml_install_signal_handler(){return 0}
+
+
+//Provides: caml_convert_raw_backtrace_slot
+//Requires: caml_failwith
+function caml_convert_raw_backtrace_slot(){
+  caml_failwith("caml_convert_raw_backtrace_slot");
+}
