@@ -18,16 +18,10 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-(*
-TODO
-- syntax highlighting?
-*)
-
 open Lwt
 open Compiler
 module Html = Dom_html
 
-
 module H = struct
   type 'a t = {
     data : 'a array;
@@ -75,7 +69,7 @@ module H = struct
       | Some t -> t
 
 
-  let load_or_create n : Js.js_string Js.t t =
+  let load_or_create n : string t =
     try
       let s = get_storage () in
       match Js.Opt.to_option (s##getItem(Js.string "history")) with
@@ -85,12 +79,12 @@ module H = struct
           if Array.length a.data = n
           then a
           else
-            let a' = make n (Js.string "") in
+            let a' = make n "" in
             for i = 0 to a.size - 1 do
               add (get a i) a';
             done;
             a'
-    with Not_found -> make n (Js.string "")
+    with _ -> make n ""
 
   let save t =
     try
@@ -101,145 +95,115 @@ module H = struct
 
 end
 
-module Top : sig
-  val setup : (string -> unit) -> unit
-  val exec : Format.formatter -> string -> unit
-  val initialize : unit -> unit
-end = struct
-  let split_primitives p =
-    let len = String.length p in
-    let rec split beg cur =
-      if cur >= len then []
-      else if p.[cur] = '\000' then
-        String.sub p beg (cur - beg) :: split (cur + 1) (cur + 1)
-      else
-        split beg (cur + 1) in
-    Array.of_list(split 0 0)
-
-  let split_char sep p =
-    let len = String.length p in
-    let rec split beg cur =
-      if cur >= len then
-        [String.sub p beg (cur - beg)]
-      else if p.[cur] = sep then
-        String.sub p beg (cur - beg) :: split (cur + 1) (cur + 1)
-      else
-        split beg (cur + 1) in
-    split 0 0
-
-  let setup output =
-    Hashtbl.add Toploop.directive_table "enable" (Toploop.Directive_string Option.Optim.enable);
-    Hashtbl.add Toploop.directive_table "disable" (Toploop.Directive_string Option.Optim.disable);
-    Hashtbl.add Toploop.directive_table "debug_on" (Toploop.Directive_string Option.Debug.enable);
-    Hashtbl.add Toploop.directive_table "debug_off" (Toploop.Directive_string Option.Debug.disable);
-    Hashtbl.add Toploop.directive_table "debug_off" (Toploop.Directive_string Option.Debug.disable);
-    Hashtbl.add Toploop.directive_table "tailcall" (Toploop.Directive_string (fun s ->
-      let x = Option.Tailcall.of_string s in
-      Option.Tailcall.set x));
-    Topdirs.dir_directory "/cmis";
-    let initial_primitive_count =
-      Array.length (split_primitives (Symtable.data_primitive_names ())) in
-
-    let compile s =
-      let prims =
-        split_primitives (Symtable.data_primitive_names ()) in
-      let unbound_primitive p =
-        try ignore (Js.Unsafe.eval_string p); false with _ -> true in
-      let stubs = ref [] in
-      Array.iteri
-        (fun i p ->
-           if i >= initial_primitive_count && unbound_primitive p then
-             stubs :=
-               Format.sprintf
-                 "function %s(){caml_failwith(\"%s not implemented\")}" p p
-               :: !stubs)
-        prims;
-      let output_program = Driver.from_string prims s in
-      let b = Buffer.create 100 in
-      output_program (Pretty_print.to_buffer b);
-      let res = Buffer.contents b in
-      let res = String.concat "" !stubs ^ res in
-      output res;
-      res
-    in
-    Js.Unsafe.global##toplevelCompile <- compile (*XXX HACK!*)
+#let_default metaocaml = false
+#if metaocaml
+let compiler_name = "MetaOCaml"
+#else
+let compiler_name = "OCaml"
+#endif
 
 
-  let refill_lexbuf s p ppf buffer len =
-    if !p = String.length s
-    then 0
-    else
-      let len',nl =
-        try String.index_from s !p '\n' - !p + 1,false
-        with _ -> String.length s - !p,true in
-      let len'' = min len len' in
-      String.blit s !p buffer 0 len'';
-      Format.fprintf ppf "%s" (String.sub buffer 0 len'');
-      if nl then Format.pp_print_newline ppf ();
-      Format.pp_print_flush ppf ();
-      p:=!p + len'';
-      len''
-
-  let exec' s =
-    let lb = Lexing.from_string s in
-    try
-      List.iter
-        (fun phr ->
-           if not (Toploop.execute_phrase false Format.std_formatter phr) then raise Exit)
-        (!Toploop.parse_use_file lb)
-    with
-    | Exit -> ()
-    | x    -> Errors.report_error Format.err_formatter x
-
-  let exec sharpf s =
-    let lb = Lexing.from_function (refill_lexbuf s (ref 0) sharpf) in
-    try
-      while true do
-        try
-          let phr = !Toploop.parse_toplevel_phrase lb in
-          ignore(Toploop.execute_phrase true Format.std_formatter phr)
-        with
-          End_of_file ->
-          raise End_of_file
-        | x ->
-          Errors.report_error Format.err_formatter x
-      done
-    with End_of_file ->
-      flush_all ()
-
-
-  (* load (binary) file from server using a synchronous XMLHttpRequest *)
-  let load_from_server path =
-    try
-      let xml = XmlHttpRequest.create () in
-      xml##_open(Js.string "GET", Js.string ("http://127.0.0.1:8888/filesys/" ^ path), Js._false);
-      xml##send(Js.null);
-      if xml##status = 200 then
-        let resp = xml##responseText in
-        let len = resp##length in
-        let str = String.create len in
-        for i=0 to len-1 do
-            str.[i] <- Char.chr (int_of_float resp##charCodeAt(i) land 0xff)
-        done;
-        Some(str)
-      else
-        None
-    with _ ->
-      None
-
-  let initialize () =
-    Sys_js.register_autoload "/" (fun s -> load_from_server s);
-    Toploop.initialize_toplevel_env ();
-    Toploop.input_name := "//toplevel//";
-    let header =
-      "        Objective Caml version %s" in
-    let header2 = Printf.sprintf
-      "     Compiled with Js_of_ocaml version %s" Sys_js.js_of_ocaml_version in
-    exec' (Printf.sprintf "Format.printf \"%s@.\" Sys.ocaml_version;;" header);
-    exec' (Printf.sprintf "Format.printf \"%s@.@.\";;" header2);
-    exec' ("#enable \"pretty\";;");
-    exec' ("#enable \"shortvar\";;")
+(* hack on *)
+module String' = struct
+  let init len f =
+    let str = Bytes.create len in
+    for i=0 to len-1 do
+      Bytes.set str i (f i)
+    done;
+    str
+  let init = (* init from String (ocaml >= 4.02) or init above *)
+    let open String in init
 end
+(* hack off *)
+
+(* load file using a synchronous XMLHttpRequest *)
+let load_resource_aux url =
+  try
+    let xml = XmlHttpRequest.create () in
+    xml##_open(Js.string "GET", Js.string url, Js._false);
+    xml##send(Js.null);
+    if xml##status = 200 then
+      let resp = xml##responseText in
+      let len = resp##length in
+      let str = String'.init len (fun i -> Char.chr (int_of_float resp##charCodeAt(i) land 0xff)) in
+      Some(str)
+    else
+      None
+  with _ ->
+    None
+
+let load_resource scheme (_,suffix) =
+  let url = Printf.sprintf "%s://%s" scheme suffix in
+  load_resource_aux url
+
+let exec' s =
+  let res : bool = JsooTop.use Format.std_formatter s in
+  if not res then Format.eprintf "error while evaluating %s@." s;
+  ()
+
+let initialize () =
+  Sys_js.register_autoload "/dev/" (fun s -> Some "");
+  Sys_js.register_autoload "/" (fun (_,s) -> load_resource_aux ("filesys/" ^ s));
+  Sys_js.register_autoload "/http/" (fun s -> load_resource "http" s);
+  Sys_js.register_autoload "/https/" (fun s -> load_resource "https" s);
+  Sys_js.register_autoload "/ftp/" (fun s -> load_resource "ftp" s);
+  JsooTop.initialize ();
+  Sys.interactive := false;
+  (* MetaOcaml *)
+#if metaocaml
+  Topdirs.dir_install_printer Format.std_formatter
+    (Longident.(Ldot(Lident "Print_code", "print_code")));
+  Topdirs.dir_install_printer Format.std_formatter
+    (Longident.(Ldot(Lident "Print_code", "print_closed_code")));
+#endif
+  exec'("let _print_error fmt e = Format.pp_print_string fmt (Js.string_of_error e)");
+  Topdirs.dir_install_printer Format.std_formatter
+    (Longident.(Lident "_print_error"));
+  Hashtbl.add Toploop.directive_table "display" (Toploop.Directive_ident (fun lid ->
+      let s =
+        match lid with
+        | Longident.Lident s -> s
+        | Longident.Ldot (_,s) -> s
+        | Longident.Lapply _ ->
+          raise Exit
+      in
+      let v : < .. > Js.t= Obj.magic (Toploop.getvalue s) in
+      if Js.instanceof v (Js.Unsafe.global ## _HTMLElement)
+      then
+        Dom.appendChild (Dom_html.getElementById "output") v
+      else
+        let s = Json.output v in
+        print_endline (Js.to_string s)
+    ));
+  exec' ("module Lwt_main = struct
+             let run t = match Lwt.state t with
+               | Lwt.Return x -> x
+               | Lwt.Fail e -> raise e
+               | Lwt.Sleep -> failwith \"Lwt_main.run: thread didn't return\"
+            end");
+  exec' ("let jsoo_logo =
+            let open Tyxml_js.Html5 in
+            a ~a:[a_href \"http://ocsigen.org/js_of_ocaml\"] [
+              img
+                ~src:\"http://ocsigen.org/resources/logos/text_js_of_ocaml_with_shadow.png\"
+                ~alt:\"Ocsigen\"  ()
+            ]");
+  exec' ("#display jsoo_logo");
+  let header1 =
+      Printf.sprintf "        %s version %%s" compiler_name in
+  let header2 = Printf.sprintf
+      "     Compiled with Js_of_ocaml version %s" Sys_js.js_of_ocaml_version in
+  exec' (Printf.sprintf "Format.printf \"%s@.\" Sys.ocaml_version;;" header1);
+  exec' (Printf.sprintf "Format.printf \"%s@.\";;" header2);
+  (if JsooTop.get_camlp4_syntaxes () <> []
+  then
+    let header3 = Printf.sprintf
+        "     'JsooTop.get_camlp4_syntaxes ()' to get loaded syntax extensions" in
+    exec' (Printf.sprintf "Format.printf \"%s@.@.\";;" header3));
+  exec' ("#enable \"pretty\";;");
+  exec' ("#enable \"shortvar\";;");
+  Sys.interactive := true;
+  ()
 
 let trim s =
   let ws c = c = ' ' || c = '\t' || c = '\n' in
@@ -252,17 +216,12 @@ let trim s =
   do decr stop done;
   String.sub s !start (!stop - !start + 1)
 
-let by_id s =
-  Js.Opt.get (Html.document##getElementById(Js.string s)) (fun () -> failwith (Printf.sprintf "cannot find dom id %S\n%!" s))
+let by_id s = Dom_html.getElementById s
 
-let by_id_coerce s f  =
-  Js.Opt.get (Js.Opt.bind (Html.document##getElementById(Js.string s)) f)
-    (fun () -> failwith (Printf.sprintf "cannot find dom id %S\n%!" s))
+let by_id_coerce s f  = Js.Opt.get (f (Dom_html.getElementById s)) (fun () -> raise Not_found)
 
 let do_by_id s f =
-  Js.Opt.case (Html.document##getElementById(Js.string s))
-    (fun () -> ())
-    (fun d -> f d)
+  try f (Dom_html.getElementById s) with Not_found -> ()
 
 let examples =
   let r = Regexp.regexp "^\\(\\*+(.*)\\*+\\)$" in
@@ -295,9 +254,10 @@ let examples =
     end
   with  _ -> []
 
-let indent_caml s in_lines =
 #let_default ocpindent = false
 #if ocpindent
+let _ = Approx_lexer.enable_extension "lwt"
+let indent_caml s in_lines =
   let output = {
     IndentPrinter.debug = false;
     config = IndentConfig.default;
@@ -310,8 +270,8 @@ let indent_caml s in_lines =
   let stream = Nstream.of_string s in
   IndentPrinter.proceed output stream IndentBlock.empty ""
 #else
+let indent_caml s in_lines = s
   (* ocp-indent not available yet in 4.02 *)
-  s
 #endif
 
 let indent_textarea textbox =
@@ -347,11 +307,32 @@ let indent_textarea textbox =
       let (lo2'',up2'') = List.assoc line2 l in
       let n1 = max (c1 + up1'' - up1) lo1'' in
       let n2 = max (c2 + up2'' - up2) lo2'' in
-      (Obj.magic textbox)##setSelectionRange(n1,n2);
+      let () = (Obj.magic textbox)##setSelectionRange(n1,n2) in
       textbox##focus();
       ()
     | None -> () end
 
+let rec filter_map f = function
+  | [] -> []
+  | x::xs ->
+    match f x with
+    | None -> filter_map f xs
+    | Some x -> x:: filter_map f xs
+
+
+(* we need to compute the hash form href to avoid different encoding behavior
+     across browser. see Url.get_fragment *)
+let parse_hash () =
+  let hash_regexp = jsnew Js.regExp (Js.string "#(.*)") in
+  let frag = Dom_html.window##location##href##_match(hash_regexp) in
+  Js.Opt.case
+    frag
+    (fun () -> [])
+    (fun res ->
+       let res = Js.match_result res in
+       let frag = Js.to_string (Js.Unsafe.get res 1) in
+       Url.decode_arguments frag)
+
 let run _ =
   let container = by_id "toplevel-container" in
   let output = by_id "output" in
@@ -364,8 +345,6 @@ let run _ =
 
   let caml_chan = open_out "/dev/null1" in
   let caml_ppf = Format.formatter_of_out_channel caml_chan in
-  let tmp = !Toploop.print_out_phrase in
-  Toploop.print_out_phrase:= (fun fmt outcome -> tmp caml_ppf outcome);
 
 
   let resize () =
@@ -376,7 +355,7 @@ let run _ =
     Lwt.return () in
 
 
-  let hist = H.load_or_create 100 in
+  let hist : string H.t = H.load_or_create 100 in
   let hist_idx = ref (H.size hist) in
   let cur = ref (Js.string "") in
   let execute () =
@@ -390,11 +369,11 @@ let run _ =
           && content.[len - 2] = ';')
       then content'
       else content' ^ ";;" in
-    H.add (Js.string content') hist;
+    H.add (content') hist;
     H.save hist;
     cur:=Js.string "";
     hist_idx:=H.size hist;
-    Top.exec sharp_ppf content;
+    JsooTop.execute true ~pp_code:sharp_ppf caml_ppf content;
     textbox##value <- Js.string "";
     resize () >>= fun () ->
     container##scrollTop <- container##scrollHeight;
@@ -402,30 +381,78 @@ let run _ =
     Lwt.return_unit in
 
   List.iter (fun (name,content) ->
-      let a = Html.createA Html.document in
-      a##className <- Js.string "list-group-item";
-      a##innerHTML <- Js.string name;
-      a##onclick <- Html.handler (fun _ ->
-          textbox##value <- Js.string content;
-          Lwt.async(fun () ->
-              resize () >>= fun () ->
-              container##scrollTop <- container##scrollHeight;
-              textbox##focus();
-              Lwt.return_unit);
-          Js._true
-        );
-      Dom.appendChild example_container a
+      let a = Tyxml_js.Html5.(a ~a:[
+          a_class ["list-group-item"];
+          a_onclick (fun _ ->
+              textbox##value <- Js.string content;
+              Lwt.async(fun () ->
+                  resize () >>= fun () ->
+                  container##scrollTop <- container##scrollHeight;
+                  textbox##focus();
+                  Lwt.return_unit);
+              true
+            )] [pcdata name]) in
+      Dom.appendChild example_container (Tyxml_js.To_dom.of_a a)
     ) examples;
 
   begin (* setup handlers *)
-    container##onclick <- Html.handler (fun _ -> textbox##focus(); Js._false);
     do_by_id "btn-execute"
       (fun e -> e##onclick <- Html.handler (fun _ -> Lwt.async execute; Js._false));
     do_by_id "btn-clear"
       (fun e -> e##onclick <- Html.handler (fun _ -> output##innerHTML <- Js.string ""; Js._false));
     do_by_id "btn-reset"
       (fun e -> e##onclick <- Html.handler (fun _ -> output##innerHTML <- Js.string "";
-                                             Top.initialize (); Js._false));
+                                             initialize (); Js._false));
+    do_by_id "btn-share"
+      (fun e ->
+         e##style##display <- Js.string "block";
+         e##onclick <- Html.handler (fun _ ->
+
+           (* get all ocaml code *)
+           let childs = Dom.list_of_nodeList output##childNodes in
+           let code = filter_map (fun t ->
+               let open Js.Opt in
+               to_option (bind (Dom_html.CoerceTo.element t) (fun t ->
+                   if Js.to_bool (t##classList##contains(Js.string "sharp"))
+                   then bind (t##textContent) (fun t -> return (Js.to_string t))
+                   else empty))) childs in
+           let code_encoded = Base64.encode (String.concat "" code) in
+
+           let url,is_file = match Url.Current.get () with
+             | Some (Url.Http url) -> Url.Http ({url with Url.hu_fragment = "" }),false
+             | Some (Url.Https url) -> Url.Https ({url with Url.hu_fragment= "" }), false
+             | Some (Url.File url) -> Url.File ({url with Url.fu_fragment= "" }), true
+             | _ -> assert false in
+
+           let frag =
+             let frags = parse_hash () in
+             let frags = List.remove_assoc "code" frags @ ["code",code_encoded] in
+             Url.encode_arguments frags
+           in
+
+           let uri = Url.string_of_url url ^ "#" ^ frag in
+           let append_url str =
+             let dom = Tyxml_js.Html5.(
+                 p [ pcdata "Share this url : "; a ~a:[a_href str] [ pcdata str ]]) in
+             Dom.appendChild output (Tyxml_js.To_dom.of_element dom);
+           in
+           Lwt.async (fun () ->
+               Lwt.catch (fun () ->
+                   if is_file
+                   then failwith "Cannot shorten url with file scheme"
+                   else
+                   let jsonp_call = Jsonp.call
+                       (Printf.sprintf "http://is.gd/create.php?format=json&url=%s" (Url.urlencode uri)) in
+                   Lwt.bind jsonp_call (fun o ->
+                       let str = Js.to_string o##shorturl in
+                       append_url str;
+                       Lwt.return_unit
+                     ))
+                 (fun exn ->
+                    Format.eprintf "Could not generate short url. reason: %s@." (Printexc.to_string exn);
+                    append_url uri;
+                    Lwt.return_unit));
+           Js._false));
 
     textbox##onkeydown <- Html.handler (fun e ->
         match e##keyCode with
@@ -454,10 +481,10 @@ let run _ =
                 let idx = !hist_idx - 1 in
                 let s=H.get hist idx in
                 hist_idx:=idx;
-                textbox##value <- s;
-                let s' = Js.to_string s in
+                textbox##value <- Js.string s;
+                let s' = s in
                 let p' = try max 0 (String.index s' '\n' - 1) with _ -> String.length s' in
-                (Obj.magic textbox)##setSelectionRange(p',p');
+                let () = (Obj.magic textbox)##setSelectionRange(p',p') in
                 Js._false
               with _ -> Js._false
           end
@@ -478,10 +505,10 @@ let run _ =
                 end else begin
                   let s=H.get hist idx in
                   hist_idx:=idx;
-                  let s' = Js.to_string s in
+                  let s' = s in
                   let slen = String.length s' in
-                  (Obj.magic textbox)##setSelectionRange(slen,slen);
-                  textbox##value <- s
+                  let () = (Obj.magic textbox)##setSelectionRange(slen,slen) in
+                  textbox##value <- Js.string s
                 end;
                 Js._false
               with _ -> Js._false
@@ -494,7 +521,7 @@ let run _ =
         | 75 when (Js.to_bool e##ctrlKey  ||
                    Js.to_bool e##altKey   ||
                    Js.to_bool e##metaKey) ->
-          Top.initialize ();
+          initialize ();
           Js._false
 
         | _ -> Js._true
@@ -505,17 +532,43 @@ let run _ =
   end;
 
   let append_string cl s =
-    let span = Html.createSpan Html.document in
-    span##className <- Js.string cl;
-    Dom.appendChild span (Html.document##createTextNode(Js.string s ));
-    Dom.appendChild output span in
+    let span = Tyxml_js.Html5.(span ~a:[a_class [cl]] [pcdata s]) in
+    Dom.appendChild output (Tyxml_js.To_dom.of_element span) in
 
-  Sys_js.set_channel_flusher caml_chan (append_string "caml");
-  Sys_js.set_channel_flusher sharp_chan (append_string "sharp");
+#let_default higlo = false
+#if higlo
+let append_ocaml cl_base s =
+  let tks = Higlo.parse ~lang:"ocaml" s in
+  let span' cl s = Tyxml_js.Html5.(span ~a:[a_class [cl]] [pcdata s]) in
+  let make_span = function
+    | Higlo.Bcomment s -> span' "comment" s
+    | Higlo.Constant s -> span' "constant" s
+    | Higlo.Directive s -> span' "directive" s
+    | Higlo.Escape s -> span' "escape" s
+    | Higlo.Id s -> span' "id" s
+    | Higlo.Keyword (level,s) -> span' (Printf.sprintf "kw%d" level) s
+    | Higlo.Lcomment s -> span' "comment" s
+    | Higlo.Numeric s -> span' "numeric" s
+    | Higlo.String s -> span' "string" s
+    | Higlo.Symbol (level,s) -> span' (Printf.sprintf "sym%d" level) s
+    | Higlo.Text s -> span' "text" s in
+  let container = Tyxml_js.Html5.(div ~a:[a_class [cl_base]] (List.map make_span tks)) in
+  Dom.appendChild output (Tyxml_js.To_dom.of_element container)
+in
+#else
+let append_ocaml = append_string in
+#endif
+  Sys_js.set_channel_flusher caml_chan (append_ocaml "caml");
+  Sys_js.set_channel_flusher sharp_chan (append_ocaml "sharp");
   Sys_js.set_channel_flusher stdout (append_string "stdout");
   Sys_js.set_channel_flusher stderr (append_string "stderr");
 
-  Lwt.async_exception_hook:=(fun exc -> Format.eprintf "exc during Lwt.async: %s@." (Printexc.to_string exc));
+  Lwt.async_exception_hook:=(fun exc ->
+    Format.eprintf "exc during Lwt.async: %s@." (Printexc.to_string exc);
+    match exc with
+    | Js.Error e -> Firebug.console##log(e##stack)
+    | _ -> ()
+ );
 
   Lwt.async (fun () ->
       resize () >>= fun () ->
@@ -523,8 +576,20 @@ let run _ =
       Lwt.return_unit
     );
   let ph = by_id "last-js" in
-
-  Top.setup (fun s -> ph##innerHTML <- Js.string s);
-  Top.initialize ()
+  initialize ();
+  let runcode : (string -> 'a) = Js.Unsafe.global##toplevelEval in
+  Js.Unsafe.global##toplevelEval <- (fun bc ->
+      ph##innerHTML <- Js.string bc;
+      runcode bc
+    );
+  (* Run initial code if any *)
+  try
+    let code = List.assoc "code" (parse_hash ()) in
+    textbox##value <- Js.string (Base64.decode code);
+    Lwt.async execute
+  with
+  | Not_found -> ()
+  | exc ->
+    Firebug.console##log_3(Js.string "exception", Js.string (Printexc.to_string exc), exc)
 
 let _ = Html.window##onload <- Html.handler (fun _ -> run (); Js._false)
